from hypothesis import given, strategies as st, assume
import pytest

from rpython.rtyper.test.test_llinterp import interpret
from rpython.translator.c.test.test_genc import compile
from rpython.tool.pytest.expecttest import ExpectTest
from rpython.tool.udir import udir
from rpython.rlib import rposix, rposix_stat, rstring
import os, sys
import errno
import py

def rposix_requires(funcname):
    return pytest.mark.skipif(not hasattr(rposix, funcname),
        reason="Requires rposix.%s()" % funcname)

win_only = pytest.mark.skipif("os.name != 'nt'")

class TestPosixFunction:
    def test_access(self):
        filename = str(udir.join('test_access.txt'))
        fd = file(filename, 'w')
        fd.close()

        for mode in os.R_OK, os.W_OK, os.X_OK, os.R_OK | os.W_OK | os.X_OK:
            result = rposix.access(filename, mode)
            assert result == os.access(filename, mode)

    def test_times(self):
        """
        posix.times should compile as an RPython function and should return a
        five-tuple giving float-representations (seconds, effectively) of the four
        fields from the underlying struct tms and the return value.
        """
        times = eval(compile(lambda: str(os.times()), ())())
        assert isinstance(times, tuple)
        assert len(times) == 5
        for value in times:
            assert isinstance(value, float)

    @py.test.mark.skipif("not hasattr(os, 'getlogin')")
    def test_getlogin(self):
        try:
            expected = os.getlogin()
        except OSError as e:
            py.test.skip("the underlying os.getlogin() failed: %s" % e)
        data = rposix.getlogin()
        assert data == expected

    @win_only
    def test_utimes(self):
        # Windows support centiseconds
        def f(fname, t1):
            os.utime(fname, (t1, t1))

        fname = udir.join('test_utimes.txt')
        fname.ensure()
        t1 = 1159195039.25
        compile(f, (str, float))(str(fname), t1)
        assert t1 == os.stat(str(fname)).st_mtime
        t1 = 5000000000.0
        compile(f, (str, float))(str(fname), t1)
        assert t1 == os.stat(str(fname)).st_mtime

    @py.test.mark.skipif("sys.platform == 'darwin'")
    def test_utime_negative_fraction(self):
        def f(fname, t1):
            os.utime(fname, (t1, t1))

        fname = udir.join('test_utime_negative_fraction.txt')
        fname.ensure()
        t1 = -123.75
        compile(f, (str, float))(str(fname), t1)
        got = os.stat(str(fname)).st_mtime
        assert got == -123 or got == -123.75

    @win_only
    def test__getfullpathname(self):
        posix = __import__(os.name)
        sysdrv = os.getenv('SystemDrive', 'C:')
        stuff = sysdrv + 'stuff'
        data = rposix.getfullpathname(stuff)
        assert data == posix._getfullpathname(stuff)
        # the most intriguing failure of ntpath.py should not repeat, here:
        assert not data.endswith(stuff)

    @win_only
    def test__getfullpathname_long(self):
        ustuff = u"C:" + u"\\abcd" * 100
        res = rposix.getfullpathname(ustuff)
        assert res == ustuff

    @win_only
    def test__getfullpathname_unicode(self):
        res = rposix.getfullpathname(u"foo\xf2\xf2")
        assert res

    def test_getcwd(self):
        assert rposix.getcwd() == os.getcwd()

    def test_chdir(self):
        def check_special_envvar():
            if sys.platform != 'win32':
                return
            pwd = os.getcwd()
            import ctypes
            buf = ctypes.create_string_buffer(1000)
            len = ctypes.windll.kernel32.GetEnvironmentVariableA('=%c:' % pwd[0], buf, 1000)
            if (len == 0) and "WINGDB_PYTHON" in os.environ:
                # the ctypes call seems not to work in the Wing debugger
                return
            assert str(buf.value).lower() == pwd.lower()
            # ctypes returns the drive letter in uppercase,
            # os.getcwd does not,
            # but there may be uppercase in os.getcwd path

        pwd = os.getcwd()
        try:
            check_special_envvar()
            rposix.chdir('..')
            assert os.getcwd() == os.path.dirname(pwd)
            check_special_envvar()
        finally:
            os.chdir(pwd)

    def test_mkdir(self):
        filename = str(udir.join('test_mkdir.dir'))
        rposix.mkdir(filename, 0777)
        with py.test.raises(OSError) as excinfo:
            rposix.mkdir(filename, 0777)
        assert excinfo.value.errno == errno.EEXIST
        if sys.platform == 'win32':
            assert excinfo.type is WindowsError

    @rposix_requires('mkdirat')
    def test_mkdirat(self):
        relpath = 'test_mkdirat.dir'
        filename = str(udir.join(relpath))
        dirfd = os.open(os.path.dirname(filename), os.O_RDONLY)
        try:
            rposix.mkdirat(relpath, 0777, dir_fd=dirfd)
            with py.test.raises(OSError) as excinfo:
                rposix.mkdirat(relpath, 0777, dir_fd=dirfd)
            assert excinfo.value.errno == errno.EEXIST
        finally:
            os.close(dirfd)

    def test_strerror(self):
        assert rposix.strerror(2) == os.strerror(2)

    def test_system(self):
        filename = str(udir.join('test_system.txt'))
        arg = '%s -c "print 1+1" > %s' % (sys.executable, filename)
        data = rposix.system(arg)
        assert data == 0
        with file(filename) as f:
            assert f.read().strip() == '2'
        os.unlink(filename)


    @py.test.mark.skipif("os.name != 'posix'")
    def test_execve(self):
        EXECVE_ENV = {"foo": "bar", "baz": "quux"}

        def run_execve(program, args=None, env=None, do_path_lookup=False):
            if args is None:
                args = [program]
            else:
                args = [program] + args
            if env is None:
                env = {}
            # we cannot directly call execve() because it replaces the
            # current process.
            fd_read, fd_write = os.pipe()
            childpid = os.fork()
            if childpid == 0:
                # in the child
                os.close(fd_read)
                os.dup2(fd_write, 1)     # stdout
                os.close(fd_write)
                if do_path_lookup:
                    os.execvp(program, args)
                else:
                    rposix.execve(program, args, env)
                assert 0, "should not arrive here"
            else:
                # in the parent
                os.close(fd_write)
                child_stdout = []
                while True:
                    data = os.read(fd_read, 4096)
                    if not data: break     # closed
                    child_stdout.append(data)
                pid, status = os.waitpid(childpid, 0)
                os.close(fd_read)
                return status, ''.join(child_stdout)

        # Test exit status and code
        result, child_stdout = run_execve("/usr/bin/which", ["true"], do_path_lookup=True)
        result, child_stdout = run_execve(child_stdout.strip()) # /bin/true or /usr/bin/true
        assert os.WIFEXITED(result)
        assert os.WEXITSTATUS(result) == 0
        result, child_stdout = run_execve("/usr/bin/which", ["false"], do_path_lookup=True)
        result, child_stdout = run_execve(child_stdout.strip()) # /bin/false or /usr/bin/false
        assert os.WIFEXITED(result)
        assert os.WEXITSTATUS(result) == 1

        # Test environment
        result, child_stdout = run_execve("/usr/bin/env", env=EXECVE_ENV)
        assert os.WIFEXITED(result)
        assert os.WEXITSTATUS(result) == 0
        assert dict([line.split('=') for line in child_stdout.splitlines()]) == EXECVE_ENV

        # The following won't actually execute anything, so they don't need
        # a child process helper.

        # If the target does not exist, an OSError should result
        info = py.test.raises(
            OSError, rposix.execve, "this/file/is/non/existent", [], {})
        assert info.value.errno == errno.ENOENT

        # If the target is not executable, an OSError should result
        info = py.test.raises(
            OSError, rposix.execve, "/etc/passwd", [], {})
        assert info.value.errno == errno.EACCES

    def test_os_write(self):
        #Same as test in rpython/test/test_rbuiltin
        fname = str(udir.join('os_test.txt'))
        fd = os.open(fname, os.O_WRONLY|os.O_CREAT, 0777)
        assert fd >= 0
        rposix.write(fd, 'Hello world')
        os.close(fd)
        with open(fname) as fid:
            assert fid.read() == "Hello world"
        fd = os.open(fname, os.O_WRONLY|os.O_CREAT, 0777)
        os.close(fd)
        py.test.raises(OSError, rposix.write, fd, 'Hello world')

    def test_os_close(self):
        fname = str(udir.join('os_test.txt'))
        fd = os.open(fname, os.O_WRONLY|os.O_CREAT, 0777)
        assert fd >= 0
        os.write(fd, 'Hello world')
        rposix.close(fd)
        py.test.raises(OSError, rposix.close, fd)

    def test_os_lseek(self):
        fname = str(udir.join('os_test.txt'))
        fd = os.open(fname, os.O_RDWR|os.O_CREAT, 0777)
        assert fd >= 0
        os.write(fd, 'Hello world')
        rposix.lseek(fd,0,0)
        assert os.read(fd, 11) == 'Hello world'
        os.close(fd)
        py.test.raises(OSError, rposix.lseek, fd, 0, 0)

    def test_os_fsync(self):
        fname = str(udir.join('os_test.txt'))
        fd = os.open(fname, os.O_WRONLY|os.O_CREAT, 0777)
        assert fd >= 0
        os.write(fd, 'Hello world')
        rposix.fsync(fd)
        os.close(fd)
        fid = open(fname)
        assert fid.read() == 'Hello world'
        fid.close()
        py.test.raises(OSError, rposix.fsync, fd)

    @py.test.mark.skipif("not hasattr(os, 'fdatasync')")
    def test_os_fdatasync(self):
        fname = str(udir.join('os_test.txt'))
        fd = os.open(fname, os.O_WRONLY|os.O_CREAT, 0777)
        assert fd >= 0
        os.write(fd, 'Hello world')
        rposix.fdatasync(fd)
        fid = open(fname)
        assert fid.read() == 'Hello world'
        os.close(fd)
        py.test.raises(OSError, rposix.fdatasync, fd)

    def test_os_kill(self):
        import subprocess
        import signal
        proc = subprocess.Popen([sys.executable, "-c",
                             "import time;"
                             "time.sleep(10)",
                             ],
                            )
        rposix.kill(proc.pid, signal.SIGTERM)
        if os.name == 'nt':
            expected = signal.SIGTERM
        else:
            expected = -signal.SIGTERM
        assert proc.wait() == expected

    def test_isatty(self):
        assert rposix.isatty(-1) is False

    @py.test.mark.skipif("not hasattr(rposix, 'makedev')")
    def test_makedev(self):
        dev = rposix.makedev(24, 7)
        assert rposix.major(dev) == 24
        assert rposix.minor(dev) == 7

    @py.test.mark.skipif("not hasattr(rposix, 'memfd_create')")
    def test_memfd_create(self):
        fd = rposix.memfd_create("abc", rposix.MFD_CLOEXEC)
        try:
            s = "defghi?"
            os.write(fd, s)
        finally:
            os.close(fd)

@py.test.mark.skipif("not hasattr(os, 'ttyname')")
class TestOsExpect(ExpectTest):
    def test_ttyname(self):
        def f():
            import os
            from rpython.rtyper.test.test_llinterp import interpret

            def ll_to_string(s):
                return ''.join(s.chars)

            def f(num):
                try:
                    return os.ttyname(num)
                except OSError:
                    return ''

            assert ll_to_string(interpret(f, [0])) == f(0)
            assert ll_to_string(interpret(f, [338])) == ''

        self.run_test(f)


def ll_to_string(s):
    return ''.join(s.chars)

class UnicodeWithEncoding:
    is_unicode = True

    def __init__(self, unistr):
        self.unistr = unistr

    def as_utf8(self):
        from rpython.rlib.runicode import unicode_encode_utf_8
        res = unicode_encode_utf_8(self.unistr, len(self.unistr), "strict")
        return rstring.assert_str0(res)

    def as_bytes(self):
        from rpython.rlib.runicode import unicode_encode_utf_8
        res = unicode_encode_utf_8(self.unistr, len(self.unistr),
                                   "strict")
        return rstring.assert_str0(res)

class BasePosixUnicodeOrAscii:
    def setup_method(self, method):
        self.ufilename = self._get_filename()
        try:
            f = file(self.ufilename, 'w')
        except UnicodeEncodeError:
            py.test.skip("encoding not good enough")
        f.write("test")
        f.close()
        if sys.platform == 'win32' and isinstance(self.ufilename, str):
            self.path = self.ufilename
            self.path2 = self.ufilename + ".new"
        else:
            self.path  = UnicodeWithEncoding(self.ufilename)
            self.path2 = UnicodeWithEncoding(self.ufilename + ".new")

    def _teardown_method(self, method):
        for path in [self.ufilename + ".new", self.ufilename]:
            if os.path.exists(path):
                os.unlink(path)

    def test_open(self):
        def f():
            try:
                fd = os.open(self.path, os.O_RDONLY, 0777)
                try:
                    text = os.read(fd, 50)
                    return text
                finally:
                    os.close(fd)
            except OSError:
                return ''

        assert ll_to_string(interpret(f, [])) == "test"

    def test_stat(self):
        def f():
            return rposix_stat.stat(self.path).st_mtime
        if sys.platform == 'win32':
            # double vs. float, be satisfied with sub-millisec resolution
            assert abs(interpret(f, []) - os.stat(self.ufilename).st_mtime) < 1e-4
        else:
            assert interpret(f, []) == os.stat(self.ufilename).st_mtime

    def test_access(self):
        def f():
            return rposix.access(self.path, os.R_OK)

        assert interpret(f, []) == 1

    def test_utime(self):
        def f():
            return rposix.utime(self.path, None)

        interpret(f, []) # does not crash

    def test_chmod(self):
        def f():
            return rposix.chmod(self.path, 0777)
        f()
        interpret(f, []) # does not crash

    def test_unlink(self):
        def f():
            return rposix.unlink(self.path)

        interpret(f, [])
        assert not os.path.exists(self.ufilename)

    def test_rename(self):
        def f():
            return rposix.rename(self.path, self.path2)

        interpret(f, [])
        assert not os.path.exists(self.ufilename)
        assert os.path.exists(self.ufilename + '.new')

    def test_replace(self):
        def f():
            return rposix.replace(self.path, self.path2)

        interpret(f, [])
        assert not os.path.exists(self.ufilename)
        assert os.path.exists(self.ufilename + '.new')

    def test_listdir(self):
        udir = UnicodeWithEncoding(os.path.dirname(self.ufilename))

        if sys.platform == 'win32':
            def f():
                contents = ', '.join(rposix.listdir(udir))
                return contents.decode('utf8')
            f()  # sanity check
            result = interpret(f, [])
            assert os.path.basename(self.ufilename) in ll_to_string(result)
        else:
            def f():
                return ', '.join(rposix.listdir(udir))
            result = interpret(f, [])
            assert (os.path.basename(self.ufilename).encode('utf-8') in
                    ll_to_string(result))

    def test_chdir(self):
        os.unlink(self.ufilename)

        def f():
            rposix.mkdir(self.path, 0777)
            rposix.chdir(self.path)

        curdir = os.getcwd()
        try:
            interpret(f, [])
            assert os.getcwdu() == os.path.realpath(self.ufilename)
        finally:
            os.chdir(curdir)

        def g():
            rposix.rmdir(self.path)

        try:
            interpret(g, [])
        finally:
            try:
                os.rmdir(self.ufilename)
            except Exception:
                pass

    def test_putenv(self):
        from rpython.rlib import rposix_environ

        def f():
            rposix.putenv(self.path, self.path)
            rposix.unsetenv(self.path)

        interpret(f, [],     # does not crash
                  malloc_check=rposix_environ.REAL_UNSETENV)
        # If we have a real unsetenv(), check that it frees the string
        # kept alive by putenv().  Otherwise, we can't check that,
        # because unsetenv() will keep another string alive itself.
    test_putenv.dont_track_allocations = True


class TestPosixAscii(BasePosixUnicodeOrAscii):
    def _get_filename(self):
        return unicode(udir.join('test_open_ascii'))

    @rposix_requires('openat')
    def test_openat(self):
        def f(dirfd):
            try:
                fd = rposix.openat('test_open_ascii', os.O_RDONLY, 0777, dirfd)
                try:
                    text = os.read(fd, 50)
                    return text
                finally:
                    os.close(fd)
            except OSError:
                return ''

        dirfd = os.open(os.path.dirname(self.ufilename), os.O_RDONLY)
        try:
            assert ll_to_string(interpret(f, [dirfd])) == "test"
        finally:
            os.close(dirfd)

    @rposix_requires('unlinkat')
    def test_unlinkat(self):
        def f(dirfd):
            return rposix.unlinkat('test_open_ascii', dir_fd=dirfd)

        dirfd = os.open(os.path.dirname(self.ufilename), os.O_RDONLY)
        try:
            interpret(f, [dirfd])
        finally:
            os.close(dirfd)
        assert not os.path.exists(self.ufilename)

    @rposix_requires('utimensat')
    def test_utimensat(self):
        def f(dirfd):
            return rposix.utimensat('test_open_ascii',
                0, rposix.UTIME_NOW, 0, rposix.UTIME_NOW, dir_fd=dirfd)

        dirfd = os.open(os.path.dirname(self.ufilename), os.O_RDONLY)
        try:
            interpret(f, [dirfd])  # does not crash
        finally:
            os.close(dirfd)

    @rposix_requires('fchmodat')
    def test_fchmodat(self):
        def f(dirfd):
            return rposix.fchmodat('test_open_ascii', 0777, dirfd)

        dirfd = os.open(os.path.dirname(self.ufilename), os.O_RDONLY)
        try:
            interpret(f, [dirfd])  # does not crash
        finally:
            os.close(dirfd)


class TestPosixUnicode(BasePosixUnicodeOrAscii):
    def _get_filename(self):
        return (unicode(udir.join('test_open')) +
                u'\u65e5\u672c.txt') # "Japan"

class TestRegisteredFunctions:
    def test_dup(self):
        def f():
            os.dup(4)
            os.dup2(5, 6)
        compile(f, ())

    def test_open(self):
        def f():
            os.open('/tmp/t', 0, 0)
            os.open(u'/tmp/t', 0, 0)
        compile(f, ())


@rposix_requires('fdlistdir')
def test_fdlistdir(tmpdir):
    tmpdir.join('file').write('text')
    dirfd = os.open(str(tmpdir), os.O_RDONLY)
    result = rposix.fdlistdir(dirfd)
    # Note: fdlistdir() always closes dirfd
    assert result == ['file']

@rposix_requires('fdlistdir')
def test_fdlistdir_rewinddir(tmpdir):
    tmpdir.join('file').write('text')
    dirfd = os.open(str(tmpdir), os.O_RDONLY)
    result1 = rposix.fdlistdir(os.dup(dirfd))
    result2 = rposix.fdlistdir(dirfd)
    assert result1 == result2 == ['file']

@rposix_requires('symlinkat')
def test_symlinkat(tmpdir):
    tmpdir.join('file').write('text')
    dirfd = os.open(str(tmpdir), os.O_RDONLY)
    try:
        rposix.symlinkat('file', 'link', dir_fd=dirfd)
        assert os.readlink(str(tmpdir.join('link'))) == 'file'
    finally:
        os.close(dirfd)

@rposix_requires('renameat')
def test_renameat(tmpdir):
    tmpdir.join('file').write('text')
    dirfd = os.open(str(tmpdir), os.O_RDONLY)
    try:
        rposix.renameat('file', 'file2', src_dir_fd=dirfd, dst_dir_fd=dirfd)
    finally:
        os.close(dirfd)
    assert tmpdir.join('file').check(exists=False)
    assert tmpdir.join('file2').check(exists=True)

def test_set_inheritable():
    fd1, fd2 = os.pipe()
    rposix.set_inheritable(fd1, True)
    assert rposix.get_inheritable(fd1) == True
    rposix.set_inheritable(fd1, False)
    assert rposix.get_inheritable(fd1) == False
    os.close(fd1)
    os.close(fd2)

def test_SetNonInheritableCache():
    cache = rposix.SetNonInheritableCache()
    fd1, fd2 = os.pipe()
    if sys.platform == 'win32':
        rposix.set_inheritable(fd1, True)
        rposix.set_inheritable(fd2, True)
    assert rposix.get_inheritable(fd1) == True
    assert rposix.get_inheritable(fd1) == True
    assert cache.cached_inheritable == -1
    cache.set_non_inheritable(fd1)
    assert cache.cached_inheritable == 1
    cache.set_non_inheritable(fd2)
    assert cache.cached_inheritable == 1
    assert rposix.get_inheritable(fd1) == False
    assert rposix.get_inheritable(fd1) == False
    os.close(fd1)
    os.close(fd2)

def test_dup_dup2_non_inheritable():
    for preset in [False, True]:
        fd1, fd2 = os.pipe()
        rposix.set_inheritable(fd1, preset)
        rposix.set_inheritable(fd2, preset)
        fd3 = rposix.dup(fd1, True)
        assert rposix.get_inheritable(fd3) == True
        fd4 = rposix.dup(fd1, False)
        assert rposix.get_inheritable(fd4) == False
        rposix.dup2(fd2, fd4, False)
        assert rposix.get_inheritable(fd4) == False
        rposix.dup2(fd2, fd3, True)
        assert rposix.get_inheritable(fd3) == True
        os.close(fd1)
        os.close(fd2)
        os.close(fd3)
        os.close(fd4)

def test_sync():
    if sys.platform != 'win32':
        rposix.sync()

def test_cpu_count():
    cc = rposix.cpu_count()
    assert cc >= 1

@rposix_requires('set_status_flags')
def test_set_status_flags():
    fd1, fd2 = os.pipe()
    try:
        flags = rposix.get_status_flags(fd1)
        assert flags & rposix.O_NONBLOCK == 0
        rposix.set_status_flags(fd1, flags | rposix.O_NONBLOCK)
        assert rposix.get_status_flags(fd1) & rposix.O_NONBLOCK != 0
    finally:
        os.close(fd1)
        os.close(fd2)

@rposix_requires('getpriority')
def test_getpriority():
    # a "don't crash" kind of test only
    prio = rposix.getpriority(rposix.PRIO_PROCESS, 0)
    rposix.setpriority(rposix.PRIO_PROCESS, 0, prio)
    py.test.raises(OSError, rposix.getpriority, rposix.PRIO_PGRP, 123456789)

if sys.platform != 'win32':
    def test_sendfile():
        from rpython.rlib import rsocket
        s1, s2 = rsocket.socketpair()
        relpath = 'test_sendfile'
        filename = str(udir.join(relpath))
        fd = os.open(filename, os.O_RDWR|os.O_CREAT, 0777)
        os.write(fd, 'abcdefghij')
        res = rposix.sendfile(s1.fd, fd, 3, 5)
        assert res == 5
        data = os.read(s2.fd, 10)
        assert data == 'defgh'
        os.close(fd)
        s2.close()
        s1.close()

    def test_sendfile_invalid_offset():
        from rpython.rlib import rsocket
        s1, s2 = rsocket.socketpair()
        relpath = 'test_sendfile_invalid_offset'
        filename = str(udir.join(relpath))
        fd = os.open(filename, os.O_RDWR|os.O_CREAT, 0777)
        os.write(fd, 'abcdefghij')
        with py.test.raises(OSError) as excinfo:
            rposix.sendfile(s1.fd, fd, -1, 5)
        assert excinfo.value.errno == errno.EINVAL
        os.close(fd)
        s2.close()
        s1.close()

if sys.platform.startswith('linux'):
    def test_sendfile_no_offset():
        from rpython.rlib import rsocket
        s1, s2 = rsocket.socketpair()
        relpath = 'test_sendfile'
        filename = str(udir.join(relpath))
        fd = os.open(filename, os.O_RDWR|os.O_CREAT, 0777)
        os.write(fd, 'abcdefghij')
        os.lseek(fd, 3, 0)
        res = rposix.sendfile_no_offset(s1.fd, fd, 5)
        assert res == 5
        data = os.read(s2.fd, 10)
        assert data == 'defgh'
        os.close(fd)
        s2.close()
        s1.close()

if sys.platform == "darwin":
   def test_sendfile_partial(tmpdir):
        # issue 3964
        from rpython.rlib.rsocket import socketpair
        wsock, rsock = socketpair()
        wsock.setblocking(False)
        rsock.setblocking(False)

        recvd = 0
        with open(str(tmpdir / "sendfile.txt"), "wb+") as file:
            file.write(b"x" * 131072)
            file.flush()
            file.seek(0)

            sent = rposix.sendfile(wsock.fd, file.fileno(),
                               offset=0, count=131072)
            assert sent > 0
            try:
                chunk = rsock.recv(1024)
                while chunk:
                    recvd += len(chunk)
                    chunk = rsock.recv(1024)
            except Exception as e:
                pass
        assert recvd  == sent


@rposix_requires('pread')
def test_pread():
    fname = str(udir.join('os_test.txt'))
    fd = os.open(fname, os.O_RDWR | os.O_CREAT)
    try:
        assert fd >= 0
        os.write(fd, b'Hello world')
        os.lseek(fd, 0, 0)
        assert rposix.pread(fd, 2, 1) == b'el'
    finally:
        os.close(fd)
    py.test.raises(OSError, rposix.pread, fd, 2, 1)

@rposix_requires('pwrite')
def test_pwrite():
    fname = str(udir.join('os_test.txt'))
    fd = os.open(fname, os.O_RDWR | os.O_CREAT, 0777)
    try:
        assert fd >= 0
        os.write(fd, b'Hello world')
        os.lseek(fd, 0, 0)
        rposix.pwrite(fd, b'ea', 1)
        assert os.read(fd, 4) == b'Heal'
    finally:
        os.close(fd)
    py.test.raises(OSError, rposix.pwrite, fd, b'ea', 1)

@rposix_requires('posix_fadvise')
def test_posix_fadvise():
    if sys.maxint <= 2**32:
        py.test.skip("ll2ctypes run of posix_fadvise() on 32-bit "
                     "gets confused by the size of OFF_T")
    fname = str(udir.join('test_os_posix_fadvise'))
    fd = os.open(fname, os.O_CREAT | os.O_RDWR)
    try:
        os.write(fd, b"foobar")
        assert rposix.posix_fadvise(fd, 0, 1, rposix.POSIX_FADV_WILLNEED) is None
        assert rposix.posix_fadvise(fd, 1, 1, rposix.POSIX_FADV_NORMAL) is None
        assert rposix.posix_fadvise(fd, 2, 1, rposix.POSIX_FADV_SEQUENTIAL) is None
        assert rposix.posix_fadvise(fd, 3, 1, rposix.POSIX_FADV_RANDOM) is None
        assert rposix.posix_fadvise(fd, 4, 1, rposix.POSIX_FADV_NOREUSE) is None
        assert rposix.posix_fadvise(fd, 5, 1, rposix.POSIX_FADV_DONTNEED) is None
        py.test.raises(OSError, rposix.posix_fadvise, fd, 6, 1, 1234567)
    finally:
        os.close(fd)

@rposix_requires('posix_fallocate')
def test_posix_fallocate():
    if sys.maxint <= 2**32:
        py.test.skip("ll2ctypes run of posix_fallocate() on 32-bit "
                     "gets confused by the size of OFF_T")
    fname = str(udir.join('os_test.txt'))
    fd = os.open(fname, os.O_WRONLY | os.O_CREAT, 0777)
    try:
        assert rposix.posix_fallocate(fd, 0, 10) == 0
    except OSError as inst:
        """ ZFS seems not to support fallocate.
        so skipping solaris-based since it is likely to come with ZFS
        """
        if inst.errno != errno.EINVAL or not sys.platform.startswith("sunos"):
            raise
    finally:
        os.close(fd)

@rposix_requires('sched_get_priority_max')
def test_sched_get_priority_max():
    assert rposix.sched_get_priority_max(rposix.SCHED_FIFO) != -1
    assert rposix.sched_get_priority_max(rposix.SCHED_RR) != -1
    assert rposix.sched_get_priority_max(rposix.SCHED_OTHER) != -1
    if sys.platform != 'darwin':
        assert rposix.sched_get_priority_max(rposix.SCHED_BATCH) != -1

@rposix_requires('sched_get_priority_min')
def test_sched_get_priority_min():
    assert rposix.sched_get_priority_min(rposix.SCHED_FIFO) != -1
    assert rposix.sched_get_priority_min(rposix.SCHED_RR) != -1
    assert rposix.sched_get_priority_min(rposix.SCHED_OTHER) != -1
    if sys.platform != 'darwin':
        assert rposix.sched_get_priority_min(rposix.SCHED_BATCH) != -1

@rposix_requires('sched_get_priority_min')
def test_os_sched_priority_max_greater_than_min():
    policy = rposix.SCHED_RR
    low = rposix.sched_get_priority_min(policy)
    high = rposix.sched_get_priority_max(policy)
    assert isinstance(low, int) == True
    assert isinstance(high, int) == True
    assert  high > low

@rposix_requires('sched_yield')
def test_sched_yield():
    if sys.platform != 'win32':
        rposix.sched_yield()

@rposix_requires('lockf')
def test_os_lockf():
    fname = str(udir.join('os_test.txt'))
    fd = os.open(fname, os.O_WRONLY | os.O_CREAT, 0777)
    try:
        os.write(fd, b'test')
        os.lseek(fd, 0, 0)
        rposix.lockf(fd, rposix.F_LOCK, 4)
        rposix.lockf(fd, rposix.F_ULOCK, 4)
    finally:
        os.close(fd)

def check_working_xattr():
    fname = str(udir.join('xattr_test0.txt'))
    with open(fname, 'wb'):
        pass
    try:
        rposix.setxattr(fname, 'user.foo', '')
    except OSError:
        return False
    else:
        return True

@pytest.mark.skipif(not (hasattr(rposix, 'getxattr') and check_working_xattr()),
    reason="Requires working rposix.getxattr()")
@given(
    name=st.text(
        alphabet=st.characters(min_codepoint=1), min_size=1, max_size=10),
    value=st.binary(max_size=10),
    follow_symlinks=st.booleans(), use_fd=st.booleans())
def test_xattr(name, value, follow_symlinks, use_fd):
    assume(follow_symlinks or not use_fd)
    name = 'user.' + name.encode('utf-8')
    fname = str(udir.join('xattr_test.txt'))
    try:
        os.unlink(fname)
    except OSError:
        pass
    with open(fname, 'wb'):
        pass
    if use_fd:
        file_id = os.open(fname, os.O_CREAT, 0777)
        read, write, delete = rposix.fgetxattr, rposix.fsetxattr, rposix.fremovexattr
        all_names = rposix.flistxattr
    else:
        file_id = fname
        if follow_symlinks:
            read, write, delete = rposix.getxattr, rposix.setxattr, rposix.removexattr
            all_names = rposix.listxattr
        else:
            read = lambda *args, **kwargs: rposix.getxattr(*args, follow_symlinks=False, **kwargs)
            write = lambda *args, **kwargs: rposix.setxattr(*args, follow_symlinks=False, **kwargs)
            delete = lambda *args, **kwargs: rposix.removexattr(*args, follow_symlinks=False, **kwargs)
            all_names = lambda *args, **kwargs: rposix.listxattr(*args, follow_symlinks=False, **kwargs)
    try:
        init_names = all_names(file_id)
        with pytest.raises(OSError):
            read(file_id, name)
        write(file_id, name, value)
        assert read(file_id, name) == value
        assert set(all_names(file_id)) == set(init_names + [name])
        assert '' not in all_names(file_id)
        delete(file_id, name)
        with pytest.raises(OSError):
            read(file_id, name)
        assert set(all_names(file_id)) == set(init_names)
    finally:
        if use_fd:
            os.close(file_id)

@pytest.mark.skipif(not hasattr(rposix, 'getgrouplist'),
    reason="Requires working rposix.getgrouplist()")
@rposix_requires('getgrouplist')
def test_getgrouplist():
    import pwd
    user = pwd.getpwuid(os.getuid())[0]
    group = pwd.getpwuid(os.getuid())[3]
    assert group in rposix.getgrouplist(user, group)

@pytest.mark.skipif(not hasattr(rposix, 'sched_rr_get_interval'),
    reason="Requires working rposix.sched_rr_get_interval()")
def test_sched_rr_get_interval():
    try:
        interval = rposix.sched_rr_get_interval(0)
    except OSError as e:
        # This likely means that sched_rr_get_interval is only valid for
        # processes with the SCHED_RR scheduler in effect.
        if e.errno != errno.EINVAL:
                raise
        pytest.mark.skip("only works on SCHED_RR processes")
    assert isinstance(interval, float)
    # Reasonable constraints, I think.
    assert interval > 0
    assert interval < 1.

@pytest.mark.skipif(not hasattr(rposix, 'sched_getscheduler'),
    reason="Requires working rposix.sched_getscheduler()")
def test_get_and_set_scheduler_and_param():
    possible_schedulers = [sched for name, sched in rposix.__dict__.items()
                                   if name.startswith("SCHED_")]
    mine = rposix.sched_getscheduler(0)
    assert mine in possible_schedulers
    try:
        parent = rposix.sched_getscheduler(os.getppid())
    except OSError as e:
        if e.errno != errno.EPERM:
            raise
    else:
        assert parent in possible_schedulers
    with pytest.raises(OSError):
        rposix.sched_getscheduler(-1)
    with pytest.raises(OSError):
        rposix.sched_getparam(-1)
    param = rposix.sched_getparam(0)
    assert param == 0

    # POSIX states that calling sched_setparam() or sched_setscheduler() on
    # a process with a scheduling policy other than SCHED_FIFO or SCHED_RR
    # is implementation-defined: NetBSD and FreeBSD can return EINVAL.
    if not sys.platform.startswith(('freebsd', 'netbsd')):
        try:
            rposix.sched_setscheduler(0, mine, param)
            rposix.sched_setparam(0, param)
        except OSError as e:
            if e.errno != errno.EPERM:
                raise
        with pytest.raises(OSError):
            rposix.sched_setparam(-1, param)
    with pytest.raises(OSError):
        rposix.sched_setscheduler(-1, mine, param)

    large = 214748364700
    if large < sys.maxint:
        param = large # rposix.sched_param(large)
        with pytest.raises(OSError):
            rposix.sched_setparam(0, param)
        # param = rposix.sched_param(sched_priority=-large)
        # with pytest.raises(OverflowError):
        #    rposix.sched_setparam(0, param)



from pypy.interpreter.mixedmodule import MixedModule 

class Module(MixedModule): 
    interpleveldefs = {
        '__name__' : '(space.wrap("mixedmodule"))',
        '__doc__'  : '(space.wrap("mixedmodule doc"))',
        'somefunc' : 'file1.somefunc', 
        'value'    : '(space.w_None)', 
        'path'     :   'file1.initpath(space)', 
        'cpypath'  : 'space.wrap(sys.path)'
    }

    appleveldefs = {
        'someappfunc' : 'file2_app.someappfunc', 
    }



from py.test import raises
from pypy.interpreter.error import OperationError
from pypy.interpreter.function import Function
from pypy.interpreter.pycode import PyCode
from rpython.rlib.rarithmetic import r_longlong, r_ulonglong
import sys

# this test isn't so much to test that the objspace interface *works*
# -- it's more to test that it's *there*


INT32_MAX = 2147483648


class TestObjSpace:

    def test_isinstance(self):
        space = self.space
        w_i = space.wrap(4)
        w_result = space.isinstance(w_i, space.w_int)
        assert space.is_true(w_result)
        assert space.isinstance_w(w_i, space.w_int)
        w_result = space.isinstance(w_i, space.w_bytes)
        assert not space.is_true(w_result)
        assert not space.isinstance_w(w_i, space.w_bytes)

    def test_newlist(self):
        w = self.space.wrap
        l = range(10)
        w_l = self.space.newlist([w(i) for i in l])
        assert self.space.eq_w(w_l, w(l))

    def test_newdict(self):
        d = {}
        w_d = self.space.newdict()
        assert self.space.eq_w(w_d, self.space.wrap(d))

    def test_newtuple(self):
        w = self.space.wrap
        t = tuple(range(10))
        w_t = self.space.newtuple([w(i) for i in t])
        assert self.space.eq_w(w_t, w(t))

    def test_is_true(self):
        w = self.space.wrap
        true = (1==1)
        false = (1==0)
        w_true = w(true)
        w_false = w(false)
        assert self.space.is_true(w_true)
        assert not self.space.is_true(w_false)

    def test_is_(self):
        w_l = self.space.newlist([])
        w_m = self.space.newlist([])
        assert self.space.is_(w_l, w_l) == self.space.w_True
        assert self.space.is_(w_l, w_m) == self.space.w_False

    def test_newbool(self):
        assert self.space.newbool(0) == self.space.w_False
        assert self.space.newbool(1) == self.space.w_True

    def test_unpackiterable(self):
        space = self.space
        w = space.wrap
        l = [space.newlist([]) for l in range(4)]
        w_l = space.newlist(l)
        l1 = space.unpackiterable(w_l)
        l2 = space.unpackiterable(w_l, 4)
        for i in range(4):
            assert space.is_w(l1[i], l[i])
            assert space.is_w(l2[i], l[i])
        err = raises(OperationError, space.unpackiterable, w_l, 3)
        assert err.value.match(space, space.w_ValueError)
        err = raises(OperationError, space.unpackiterable, w_l, 5)
        assert err.value.match(space, space.w_ValueError)
        w_a = space.appexec((), """():
        class A(object):
            def __iter__(self):
                return self
            def next(self):
                raise StopIteration
            def __len__(self):
                1/0
        return A()
        """)
        try:
            space.unpackiterable(w_a)
        except OperationError as o:
            if not o.match(space, space.w_ZeroDivisionError):
                raise Exception("DID NOT RAISE")
        else:
            raise Exception("DID NOT RAISE")

    def test_fixedview(self):
        space = self.space
        w = space.wrap
        l = [w(1), w(2), w(3), w(4)]
        w_l = space.newtuple(l)
        assert space.fixedview(w_l) == l
        assert space.fixedview(w_l, 4) == l
        err = raises(OperationError, space.fixedview, w_l, 3)
        assert err.value.match(space, space.w_ValueError)
        err = raises(OperationError, space.fixedview, w_l, 5)
        assert err.value.match(space, space.w_ValueError)

    def test_listview(self):
        space = self.space
        w = space.wrap
        l = [w(1), w(2), w(3), w(4)]
        w_l = space.newtuple(l)
        assert space.listview(w_l) == l
        assert space.listview(w_l, 4) == l
        err = raises(OperationError, space.listview, w_l, 3)
        assert err.value.match(space, space.w_ValueError)
        err = raises(OperationError, space.listview, w_l, 5)
        assert err.value.match(space, space.w_ValueError)

    def test_exception_match(self):
        assert self.space.exception_match(self.space.w_ValueError,
                                                   self.space.w_ValueError)
        assert self.space.exception_match(self.space.w_IndexError,
                                                   self.space.w_LookupError)
        assert not self.space.exception_match(self.space.w_ValueError,
                                               self.space.w_LookupError)

    def test_lookup(self):
        w = self.space.wrap
        w_object_doc = self.space.getattr(self.space.w_object, w("__doc__"))
        w_instance = self.space.appexec([], "(): return object()")
        assert self.space.lookup(w_instance, "__doc__") == w_object_doc 
        assert self.space.lookup(w_instance, "gobbledygook") is None
        w_instance = self.space.appexec([], """():
            class Lookup(object):
                "bla" 
            return Lookup()""")
        assert self.space.str_w(self.space.lookup(w_instance, "__doc__")) == "bla"

    def test_callable(self):
        def is_callable(w_obj):
            return self.space.is_true(self.space.callable(w_obj))

        assert is_callable(self.space.w_int)
        assert not is_callable(self.space.wrap(1))
        w_func = self.space.appexec([], "():\n def f(): pass\n return f")
        assert is_callable(w_func)
        w_lambda_func = self.space.appexec([], "(): return lambda: True")
        assert is_callable(w_lambda_func)
        
        w_instance = self.space.appexec([], """():
            class Call(object):
                def __call__(self): pass
            return Call()""")
        assert is_callable(w_instance)

        w_instance = self.space.appexec([],
                "():\n class NoCall(object): pass\n return NoCall()")
        assert not is_callable(w_instance)
        self.space.setattr(w_instance, self.space.wrap("__call__"), w_func)
        assert not is_callable(w_instance)

        w_oldstyle = self.space.appexec([], """():
            class NoCall:
                pass
            return NoCall()""")
        assert not is_callable(w_oldstyle)
        self.space.setattr(w_oldstyle, self.space.wrap("__call__"), w_func)
        assert is_callable(w_oldstyle)

    def test_int_w(self):
        space = self.space
        w_x = space.wrap(42)
        assert space.int_w(w_x) == 42
        assert space.int_w(w_x, allow_conversion=False) == 42
        #
        w_x = space.wrap(44.0)
        space.raises_w(space.w_TypeError, space.int_w, w_x)
        space.raises_w(space.w_TypeError, space.int_w, w_x, allow_conversion=False)
        #
        w_instance = self.space.appexec([], """():
            class MyInt(object):
                def __int__(self):
                    return 43
            return MyInt()
        """)
        assert space.int_w(w_instance) == 43
        space.raises_w(space.w_TypeError, space.int_w, w_instance, allow_conversion=False)
        #
        w_instance = self.space.appexec([], """():
            class MyInt(object):
                def __int__(self):
                    return 43

            class AnotherInt(object):
                def __int__(self):
                    return MyInt()

            return AnotherInt()
        """)
        space.raises_w(space.w_TypeError, space.int_w, w_instance)
        space.raises_w(space.w_TypeError, space.int_w, w_instance, allow_conversion=False)


    def test_interp_w(self):
        w = self.space.wrap
        w_bltinfunction = self.space.builtin.get('len')
        res = self.space.interp_w(Function, w_bltinfunction)
        assert res is w_bltinfunction   # with the std objspace only
        self.space.raises_w(self.space.w_TypeError, self.space.interp_w, PyCode, w_bltinfunction)
        self.space.raises_w(self.space.w_TypeError, self.space.interp_w, Function, w(42))
        self.space.raises_w(self.space.w_TypeError, self.space.interp_w, Function, w(None))
        res = self.space.interp_w(Function, w(None), can_be_None=True)
        assert res is None

    def test_text0_w(self):
        space = self.space
        w = space.wrap
        assert space.text0_w(w("123")) == "123"
        space.raises_w(space.w_TypeError, space.text0_w, w("123\x004"))

    def test_getindex_w(self):
        w_instance1 = self.space.appexec([], """():
            class X(object):
                def __index__(self): return 42
            return X()""")
        w_instance2 = self.space.appexec([], """():
            class Y(object):
                def __index__(self): return 2**70
            return Y()""")
        first = self.space.getindex_w(w_instance1, None)
        assert first == 42
        second = self.space.getindex_w(w_instance2, None)
        assert second == sys.maxint
        self.space.raises_w(self.space.w_IndexError,
                            self.space.getindex_w, w_instance2, self.space.w_IndexError)
        try:
            self.space.getindex_w(self.space.w_tuple, None, "foobar")
        except OperationError as e:
            assert e.match(self.space, self.space.w_TypeError)
            assert "foobar" in e.errorstr(self.space)
        else:
            assert 0, "should have raised"

    def test_r_longlong_w(self):
        space = self.space
        w_value = space.wrap(12)
        res = space.r_longlong_w(w_value)
        assert res == 12
        assert type(res) is r_longlong
        w_value = space.wrap(r_longlong(-INT32_MAX * 42))
        res = space.r_longlong_w(w_value)
        assert res == -INT32_MAX * 42
        assert type(res) is r_longlong
        w_obj = space.wrap("hello world")
        space.raises_w(space.w_TypeError, space.r_longlong_w, w_obj)
        w_obj = space.wrap(-12.34)
        space.raises_w(space.w_TypeError, space.r_longlong_w, w_obj)

    def test_r_ulonglong_w(self):
        space = self.space
        w_value = space.wrap(12)
        res = space.r_ulonglong_w(w_value)
        assert res == 12
        assert type(res) is r_ulonglong
        w_value = space.wrap(r_ulonglong(INT32_MAX * 42))
        res = space.r_ulonglong_w(w_value)
        assert res == INT32_MAX * 42
        assert type(res) is r_ulonglong
        w_obj = space.wrap("hello world")
        space.raises_w(space.w_TypeError, space.r_ulonglong_w, w_obj)
        w_obj = space.wrap(-12.34)
        space.raises_w(space.w_TypeError, space.r_ulonglong_w, w_obj)
        w_obj = space.wrap(-12)
        space.raises_w(space.w_ValueError, space.r_ulonglong_w, w_obj)

    def test_truncatedint_w(self):
        space = self.space
        assert space.truncatedint_w(space.wrap(42)) == 42
        assert space.truncatedint_w(space.wrap(sys.maxint)) == sys.maxint
        assert space.truncatedint_w(space.wrap(sys.maxint+1)) == -sys.maxint-1
        assert space.truncatedint_w(space.wrap(-1)) == -1
        assert space.truncatedint_w(space.wrap(-sys.maxint-2)) == sys.maxint

    def test_truncatedlonglong_w(self):
        space = self.space
        w_value = space.wrap(12)
        res = space.truncatedlonglong_w(w_value)
        assert res == 12
        assert type(res) is r_longlong
        #
        w_value = space.wrap(r_ulonglong(9223372036854775808))
        res = space.truncatedlonglong_w(w_value)
        assert res == -9223372036854775808
        assert type(res) is r_longlong
        #
        w_value = space.wrap(r_ulonglong(18446744073709551615))
        res = space.truncatedlonglong_w(w_value)
        assert res == -1
        assert type(res) is r_longlong
        #
        w_value = space.wrap(r_ulonglong(18446744073709551616))
        res = space.truncatedlonglong_w(w_value)
        assert res == 0
        assert type(res) is r_longlong


    def test_call_obj_args(self):
        from pypy.interpreter.argument import Arguments
        
        space = self.space

        w_f = space.appexec([], """():
    def f(x, y):
        return (x, y)
    return f
""")

        w_a = space.appexec([], """():
    class A(object):
        def __call__(self, x):
            return x
    return A()
""")

        w_9 = space.wrap(9)
        w_1 = space.wrap(1)

        w_res = space.call_obj_args(w_f, w_9, Arguments(space, [w_1]))

        w_x, w_y = space.fixedview(w_res, 2)
        assert w_x is w_9
        assert w_y is w_1

        w_res = space.call_obj_args(w_a, w_9, Arguments(space, []))        
        assert w_res is w_9

    def test_compare_by_iteration(self):
        import operator
        space = self.space
        for op in ['eq', 'ne', 'lt', 'le', 'gt', 'ge']:
            comparer = getattr(operator, op)
            for x in [[], [0], [0, 1], [0, 1, 2]]:
                for y in [[], [-1], [0], [1], [-1, 0],
                          [0, 0], [0, 1], [0, 2],
                          [0, 1, 1], [0, 1, 2], [0, 1, 3]]:
                        w_res = space.compare_by_iteration(space.wrap(x),
                                                           space.wrap(y), op)
                        if comparer(x, y):
                            assert w_res is space.w_True
                        else:
                            assert w_res is space.w_False

class TestModuleMinimal: 
    def test_sys_exists(self):
        assert self.space.sys

    def test_import_exists(self):
        space = self.space
        assert space.builtin
        w_name = space.wrap('__import__')
        w_builtin = space.sys.getmodule('__builtin__')
        w_import = self.space.getattr(w_builtin, w_name)
        assert space.is_true(w_import)

    def test_sys_import(self):
        from pypy.interpreter.main import run_string
        run_string('import sys', space=self.space)

    def test_dont_reload_builtin_mods_on_startup(self):
        from pypy.tool.option import make_config, make_objspace
        config = make_config(None)
        space = make_objspace(config)
        w_executable = space.wrap('executable')
        assert space.findattr(space.sys, w_executable) is None
        space.setattr(space.sys, w_executable, space.wrap('foobar'))
        assert space.str_w(space.getattr(space.sys, w_executable)) == 'foobar'
        space.startup()
        assert space.str_w(space.getattr(space.sys, w_executable)) == 'foobar'

    def test_interned_strings_are_weak(self):
        import weakref, gc, random
        space = self.space
        assert space.config.translation.rweakref
        w1 = space.new_interned_str("abcdef")
        w2 = space.new_interned_str("abcdef")
        assert w2 is w1
        #
        # check that 'w1' goes away if we don't hold a reference to it
        rw1 = weakref.ref(w1)
        del w1, w2
        i = 10
        while rw1() is not None:
            i -= 1
            assert i >= 0
            gc.collect()
        #
        s = "foobar%r" % random.random()
        w0 = space.wrap(s)
        w1 = space.new_interned_w_str(w0)
        assert w1 is w0
        w2 = space.new_interned_w_str(w0)
        assert w2 is w0

    def test_exitfunc_catches_exceptions(self):
        from pypy.tool.pytest.objspace import maketestobjspace
        space = maketestobjspace()
        space.appexec([], """():
            import sys
            sys.exitfunc = lambda: this_is_an_unknown_name
        """)
        space.finish()
        # assert that we reach this point without getting interrupted
        # by the OperationError(NameError)

    def test_format_traceback(self):
        from pypy.tool.pytest.objspace import maketestobjspace
        from pypy.interpreter.gateway import interp2app
        #
        def format_traceback(space):
            return space.format_traceback()
        #
        space = maketestobjspace()
        w_format_traceback = space.wrap(interp2app(format_traceback))
        w_tb = space.appexec([w_format_traceback], """(format_traceback):
            def foo():
                return bar()
            def bar():
                return format_traceback()
            return foo()
        """)
        tb = space.str_w(w_tb)
        expected = '\n'.join([
            '  File "?", line 6, in anonymous',  # this is the appexec code object
            '  File "?", line 3, in foo',
            '  File "?", line 5, in bar',
            ''
        ])
        assert tb == expected



"""
Implementation of the 'abstract instance and subclasses' protocol:
objects can return pseudo-classes as their '__class__' attribute, and
pseudo-classes can have a '__bases__' attribute with a tuple of other
pseudo-classes.  The standard built-in functions isinstance() and
issubclass() follow and trust these attributes is they are present, in
addition to checking for instances and subtypes in the normal way.
"""

from rpython.rlib import jit

from pypy.interpreter.baseobjspace import ObjSpace as BaseObjSpace
from pypy.interpreter.error import OperationError
from pypy.module.__builtin__.interp_classobj import W_ClassObject
from pypy.module.__builtin__.interp_classobj import W_InstanceObject


def _get_bases(space, w_cls):
    """Returns 'cls.__bases__'.  Returns None if there is
    no __bases__ or if cls.__bases__ is not a tuple.
    """
    try:
        w_bases = space.getattr(w_cls, space.newtext('__bases__'))
    except OperationError as e:
        if not e.match(space, space.w_AttributeError):
            raise       # propagate other errors
        return None
    if space.isinstance_w(w_bases, space.w_tuple):
        return w_bases
    else:
        return None

def abstract_isclass_w(space, w_obj):
    return _get_bases(space, w_obj) is not None

def check_class(space, w_obj, msg):
    if not abstract_isclass_w(space, w_obj):
        raise OperationError(space.w_TypeError, space.newtext(msg))


def abstract_getclass(space, w_obj):
    try:
        return space.getattr(w_obj, space.newtext('__class__'))
    except OperationError as e:
        if not e.match(space, space.w_AttributeError):
            raise       # propagate other errors
        return space.type(w_obj)


# ---------- isinstance ----------


def p_recursive_isinstance_w(space, w_inst, w_cls):
    # Copied straight from CPython 2.7.  Does not handle 'cls' being a tuple.
    if (isinstance(w_cls, W_ClassObject) and
        isinstance(w_inst, W_InstanceObject)):
        return w_inst.w_class.is_subclass_of(w_cls)

    if space.isinstance_w(w_cls, space.w_type):
        return p_recursive_isinstance_type_w(space, w_inst, w_cls)

    check_class(space, w_cls, "isinstance() arg 2 must be a class, type,"
                              " or tuple of classes and types")
    try:
        w_abstractclass = space.getattr(w_inst, space.newtext('__class__'))
    except OperationError as e:
        if e.async(space):      # ignore most exceptions
            raise
        return False
    else:
        return p_abstract_issubclass_w(space, w_abstractclass, w_cls)


def p_recursive_isinstance_type_w(space, w_inst, w_type):
    # subfunctionality of p_recursive_isinstance_w(): assumes that w_type is
    # a type object.  Copied straight from CPython 2.7.
    if space.isinstance_w(w_inst, w_type):
        return True
    try:
        w_abstractclass = space.getattr(w_inst, space.newtext('__class__'))
    except OperationError as e:
        if e.async(space):      # ignore most exceptions
            raise
    else:
        if w_abstractclass is not space.type(w_inst):
            if space.isinstance_w(w_abstractclass, space.w_type):
                return space.issubtype_w(w_abstractclass, w_type)
    return False


@jit.unroll_safe
def abstract_isinstance_w(space, w_obj, w_klass_or_tuple, allow_override=False):
    """Implementation for the full 'isinstance(obj, klass_or_tuple)'."""
    # Copied from CPython 2.7's PyObject_Isinstance().  Additionally,
    # if 'allow_override' is False (the default), then don't try to
    # use a custom __instancecheck__ method.

    # WARNING: backward compatibility function name here.  CPython
    # uses the name "abstract" to refer to the logic of handling
    # class-like objects, with a "__bases__" attribute.  This function
    # here is not related to that and implements the full
    # PyObject_IsInstance() logic.

    # Quick test for an exact match
    if space.type(w_obj) is w_klass_or_tuple:
        return True

    # -- case (anything, tuple)
    # XXX it might be risky that the JIT sees this
    if space.isinstance_w(w_klass_or_tuple, space.w_tuple):
        for w_klass in space.fixedview(w_klass_or_tuple):
            if abstract_isinstance_w(space, w_obj, w_klass, allow_override):
                return True
        return False

    # -- case (anything, type)
    if allow_override:
        w_check = space.lookup(w_klass_or_tuple, "__instancecheck__")
        if w_check is not None:
            # this is the common case: all type objects have a method
            # __instancecheck__.  The one in the base 'type' type calls
            # back p_recursive_isinstance_type_w() from the present module.
            return space.is_true(space.get_and_call_function(
                w_check, w_klass_or_tuple, w_obj))

    return p_recursive_isinstance_w(space, w_obj, w_klass_or_tuple)


# ---------- issubclass ----------


@jit.unroll_safe
def p_abstract_issubclass_w(space, w_derived, w_cls):
    # Copied straight from CPython 2.7, function abstract_issubclass().
    # Don't confuse this with the function abstract_issubclass_w() below.
    # Here, w_cls cannot be a tuple.
    while True:
        if space.is_w(w_derived, w_cls):
            return True
        w_bases = _get_bases(space, w_derived)
        if w_bases is None:
            return False
        bases_w = space.fixedview(w_bases)
        last_index = len(bases_w) - 1
        if last_index < 0:
            return False
        # Avoid recursivity in the single inheritance case; in general,
        # don't recurse on the last item in the tuple (loop instead).
        for i in range(last_index):
            if p_abstract_issubclass_w(space, bases_w[i], w_cls):
                return True
        w_derived = bases_w[last_index]


def p_recursive_issubclass_w(space, w_derived, w_cls):
    # From CPython's function of the same name (which as far as I can tell
    # is not recursive).  Copied straight from CPython 2.7.
    if (space.isinstance_w(w_cls, space.w_type) and
        space.isinstance_w(w_derived, space.w_type)):
        return space.issubtype_w(w_derived, w_cls)
    #
    if (isinstance(w_derived, W_ClassObject) and
        isinstance(w_cls, W_ClassObject)):
        return w_derived.is_subclass_of(w_cls)
    #
    check_class(space, w_derived, "issubclass() arg 1 must be a class")
    check_class(space, w_cls, "issubclass() arg 2 must be a class"
                              " or tuple of classes")
    return p_abstract_issubclass_w(space, w_derived, w_cls)


@jit.unroll_safe
def abstract_issubclass_w(space, w_derived, w_klass_or_tuple,
                          allow_override=False):
    """Implementation for the full 'issubclass(derived, klass_or_tuple)'."""

    # WARNING: backward compatibility function name here.  CPython
    # uses the name "abstract" to refer to the logic of handling
    # class-like objects, with a "__bases__" attribute.  This function
    # here is not related to that and implements the full
    # PyObject_IsSubclass() logic.  There is also p_abstract_issubclass_w().

    # -- case (anything, tuple-of-classes)
    if space.isinstance_w(w_klass_or_tuple, space.w_tuple):
        for w_klass in space.fixedview(w_klass_or_tuple):
            if abstract_issubclass_w(space, w_derived, w_klass, allow_override):
                return True
        return False

    # -- case (anything, type)
    if allow_override:
        w_check = space.lookup(w_klass_or_tuple, "__subclasscheck__")
        if w_check is not None:
            # this is the common case: all type objects have a method
            # __subclasscheck__.  The one in the base 'type' type calls
            # back p_recursive_issubclass_w() from the present module.
            return space.is_true(space.get_and_call_function(
                w_check, w_klass_or_tuple, w_derived))

    return p_recursive_issubclass_w(space, w_derived, w_klass_or_tuple)


# ------------------------------------------------------------
# Exception helpers

def exception_is_valid_obj_as_class_w(space, w_obj):
    if isinstance(w_obj, W_ClassObject):
        return True
    return BaseObjSpace.exception_is_valid_obj_as_class_w(space, w_obj)

def exception_is_valid_class_w(space, w_cls):
    if isinstance(w_cls, W_ClassObject):
        return True
    return BaseObjSpace.exception_is_valid_class_w(space, w_cls)

def exception_getclass(space, w_obj):
    if isinstance(w_obj, W_InstanceObject):
        return w_obj.w_class
    return BaseObjSpace.exception_getclass(space, w_obj)

def exception_issubclass_w(space, w_cls1, w_cls2):
    if (space.type(w_cls1) is space.w_type and
        space.type(w_cls2) is space.w_type):
        return BaseObjSpace.exception_issubclass_w(space, w_cls1, w_cls2)
    #
    if (not exception_is_valid_class_w(space, w_cls2) or
        not exception_is_valid_class_w(space, w_cls1)):
        return False
    #
    # The rest is the rare slow case.  Use the general logic of issubclass()
    # (issue #3149).  CPython 3.x doesn't do that (but there is a
    # many-years issue report: https://bugs.python.org/issue12029), and
    # there are probably tests, so we won't call abstract_issubclass_w()
    # either in PyPy3.
    try:
        return abstract_issubclass_w(space, w_cls1, w_cls2, True)
    except OperationError as e:
        if e.async(space):
            raise
        return False

# ____________________________________________________________
# App-level interface

def issubclass(space, w_cls, w_klass_or_tuple):
    """Check whether a class 'cls' is a subclass (i.e., a derived class) of
another class.  When using a tuple as the second argument, check whether
'cls' is a subclass of any of the classes listed in the tuple."""
    result = abstract_issubclass_w(space, w_cls, w_klass_or_tuple, True)
    return space.newbool(result)

def isinstance(space, w_obj, w_klass_or_tuple):
    """Check whether an object is an instance of a class (or of a subclass
thereof).  When using a tuple as the second argument, check whether 'obj'
is an instance of any of the classes listed in the tuple."""
    result = abstract_isinstance_w(space, w_obj, w_klass_or_tuple, True)
    return space.newbool(result)

# avoid namespace pollution
app_issubclass = issubclass; del issubclass
app_isinstance = isinstance; del isinstance



"""
These tests are supposed to run on the following platforms:
1. CStackless
2. CPython (with the stackless_new module in the path
3. pypy-c
"""
from __future__ import absolute_import
from py.test import skip
try:
    import stackless
except ImportError:
    try:
        from lib_pypy import stackless
    except ImportError as e:
        skip('cannot import stackless: %s' % (e,))

SHOW_STRANGE = False

def dprint(txt):
    if SHOW_STRANGE:
        print txt

class Test_Stackless:

    def test_simple(self):
        rlist = []

        def f():
            rlist.append('f')

        def g():
            rlist.append('g')
            stackless.schedule()

        def main():
            rlist.append('m')
            cg = stackless.tasklet(g)()
            cf = stackless.tasklet(f)()
            stackless.run()
            rlist.append('m')

        main()

        assert stackless.getcurrent() is stackless.getmain()
        assert rlist == 'm g f m'.split()

    def test_with_channel(self):
        pref = {}
        pref[-1] = ['s0', 'r0', 's1', 'r1', 's2', 'r2', 
                    's3', 'r3', 's4', 'r4', 's5', 'r5', 
                    's6', 'r6', 's7', 'r7', 's8', 'r8', 
                    's9', 'r9']
        pref[0] =  ['s0', 'r0', 's1', 's2', 'r1', 'r2', 
                    's3', 's4', 'r3', 'r4', 's5', 's6', 
                    'r5', 'r6', 's7', 's8', 'r7', 'r8', 
                    's9', 'r9']
        pref[1] =  ['s0', 's1', 'r0', 's2', 'r1', 's3', 
                    'r2', 's4', 'r3', 's5', 'r4', 's6', 
                    'r5', 's7', 'r6', 's8', 'r7', 's9', 
                    'r8', 'r9']
        rlist = []

        def f(outchan):
            for i in range(10):
                rlist.append('s%s' % i)
                outchan.send(i)
            outchan.send(-1)

        def g(inchan):
            while 1:
                val = inchan.receive()
                if val == -1:
                    break
                rlist.append('r%s' % val)

        for preference in [-1, 0, 1]:
            rlist = []
            ch = stackless.channel()
            ch.preference = preference
            t1 = stackless.tasklet(f)(ch)
            t2 = stackless.tasklet(g)(ch)

            stackless.run()

            assert len(rlist) == 20
            assert rlist == pref[preference]

    def test_send_counter(self):
        import random

        numbers = range(20)
        random.shuffle(numbers)

        def counter(n, ch):
            for i in xrange(n):
                stackless.schedule()
            ch.send(n)

        ch = stackless.channel()
        for each in numbers:
            stackless.tasklet(counter)(each, ch)

        stackless.run()

        rlist = []
        while ch.balance:
            rlist.append(ch.receive())

        numbers.sort()
        assert rlist == numbers

    def test_receive_counter(self):
        import random

        numbers = range(20)
        random.shuffle(numbers)

        rlist = []
        def counter(n, ch):
            for i in xrange(n):
                stackless.schedule()
            ch.receive()
            rlist.append(n)

        ch = stackless.channel()
        for each in numbers:
            stackless.tasklet(counter)(each, ch)

        stackless.run()

        while ch.balance:
            ch.send(None)

        numbers.sort()
        assert rlist == numbers

    def test_scheduling_cleanup(self):
        rlist = []
        def f():
            rlist.append('fb')
            stackless.schedule()
            rlist.append('fa')

        def g():
            rlist.append('gb')
            stackless.schedule()
            rlist.append('ga')

        def h():
            rlist.append('hb')
            stackless.schedule()
            rlist.append('ha')

        tf = stackless.tasklet(f)()
        tg = stackless.tasklet(g)()
        th = stackless.tasklet(h)()

        rlist.append('mb')
        stackless.run()
        rlist.append('ma')

        assert rlist == 'mb fb gb hb fa ga ha ma'.split()

    def test_except(self):
        rlist = []
        def f():
            rlist.append('f')
            return 1/0

        def g():
            rlist.append('bg')
            stackless.schedule()
            rlist.append('ag')

        def h():
            rlist.append('bh')
            stackless.schedule()
            rlist.append('ah')

        tg = stackless.tasklet(g)()
        tf = stackless.tasklet(f)()
        th = stackless.tasklet(h)()

        try:
            stackless.run()
        # cheating, can't test for ZeroDivisionError
        except Exception as e:
            rlist.append('E')
        stackless.schedule()
        stackless.schedule()

        assert rlist == "bg f E bh ag ah".split()

    def test_except_full(self):
        rlist = []
        def f():
            rlist.append('f')
            return 1/0

        def g():
            rlist.append('bg')
            stackless.schedule()
            rlist.append('ag')

        def h():
            rlist.append('bh')
            stackless.schedule()
            rlist.append('ah')

        tg = stackless.tasklet(g)()
        tf = stackless.tasklet(f)()
        th = stackless.tasklet(h)()

        try:
            stackless.run()
        except ZeroDivisionError:
            rlist.append('E')
        stackless.schedule()
        stackless.schedule()

        assert rlist == "bg f E bh ag ah".split()

    def test_kill(self):
        def f():pass
        t =  stackless.tasklet(f)()
        t.kill()
        assert not t.alive

    def test_catch_taskletexit(self):
        # Tests if TaskletExit can be caught in the tasklet being killed.
        global taskletexit
        taskletexit = False
        
        def f():
            try:
                stackless.schedule()
            except TaskletExit:
                global TaskletExit
                taskletexit = True
                raise
            
            t =  stackless.tasklet(f)()
            t.run()
            assert t.alive
            t.kill()
            assert not t.alive
            assert taskletexit
            
    def test_autocatch_taskletexit(self):
        # Tests if TaskletExit is caught correctly in stackless.tasklet.setup(). 
        def f():
            stackless.schedule()
        
        t = stackless.tasklet(f)()
        t.run()
        t.kill()


    # tests inspired from simple stackless.com examples

    def test_construction(self):
        output = []
        def print_(*args):
            output.append(args)

        def aCallable(value):
            print_("aCallable:", value)

        task = stackless.tasklet(aCallable)
        task.setup('Inline using setup')

        stackless.run()
        assert output == [("aCallable:", 'Inline using setup')]


        del output[:]
        task = stackless.tasklet(aCallable)
        task('Inline using ()')

        stackless.run()
        assert output == [("aCallable:", 'Inline using ()')]
        
        del output[:]
        task = stackless.tasklet()
        task.bind(aCallable)
        task('Bind using ()')

        stackless.run()
        assert output == [("aCallable:", 'Bind using ()')]

    def test_simple_channel(self):
        output = []
        def print_(*args):
            output.append(args)
            
        def Sending(channel):
            print_("sending")
            channel.send("foo")

        def Receiving(channel):
            print_("receiving")
            print_(channel.receive())

        ch=stackless.channel()

        task=stackless.tasklet(Sending)(ch)

        # Note: the argument, schedule is taking is the value,
        # schedule returns, not the task that runs next

        #stackless.schedule(task)
        stackless.schedule()
        task2=stackless.tasklet(Receiving)(ch)
        #stackless.schedule(task2)
        stackless.schedule()

        stackless.run()

        assert output == [('sending',), ('receiving',), ('foo',)]

    def test_balance_zero(self):
        ch=stackless.channel()
        assert ch.balance == 0
        
    def test_balance_send(self):
        def Sending(channel):
            channel.send("foo")

        ch=stackless.channel()

        task=stackless.tasklet(Sending)(ch)
        stackless.run()

        assert ch.balance == 1

    def test_balance_recv(self):
        def Receiving(channel):
            channel.receive()

        ch=stackless.channel()

        task=stackless.tasklet(Receiving)(ch)
        stackless.run()

        assert ch.balance == -1

    def test_run(self):
        output = []
        def print_(*args):
            output.append(args)

        def f(i):
            print_(i)

        stackless.tasklet(f)(1)
        stackless.tasklet(f)(2)
        stackless.run()

        assert output == [(1,), (2,)]

    def test_schedule(self):
        output = []
        def print_(*args):
            output.append(args)

        def f(i):
            print_(i)

        stackless.tasklet(f)(1)
        stackless.tasklet(f)(2)
        stackless.schedule()

        assert output == [(1,), (2,)]


    def test_cooperative(self):
        output = []
        def print_(*args):
            output.append(args)

        def Loop(i):
            for x in range(3):
                stackless.schedule()
                print_("schedule", i)

        stackless.tasklet(Loop)(1)
        stackless.tasklet(Loop)(2)
        stackless.run()

        assert output == [('schedule', 1), ('schedule', 2),
                          ('schedule', 1), ('schedule', 2),
                          ('schedule', 1), ('schedule', 2),]

    def test_channel_callback(self):
        res = []
        cb = []
        def callback_function(chan, task, sending, willblock):
            cb.append((chan, task, sending, willblock))
        stackless.set_channel_callback(callback_function)
        def f(chan):
            chan.send('hello')
            val = chan.receive()
            res.append(val)

        chan = stackless.channel()
        task = stackless.tasklet(f)(chan)
        val = chan.receive()
        res.append(val)
        chan.send('world')
        assert res == ['hello','world']
        maintask = stackless.getmain()
        assert cb == [
            (chan, maintask, 0, 1), 
            (chan, task, 1, 0), 
            (chan, maintask, 1, 1), 
            (chan, task, 0, 0)
        ]

    def test_schedule_callback(self):
        res = []
        cb = []
        def schedule_cb(prev, next):
            cb.append((prev, next))

        stackless.set_schedule_callback(schedule_cb)
        def f(i):
            res.append('A_%s' % i)
            stackless.schedule()
            res.append('B_%s' % i)

        t1 = stackless.tasklet(f)(1)
        t2 = stackless.tasklet(f)(2)
        maintask = stackless.getmain()
        stackless.run()
        assert res == ['A_1', 'A_2', 'B_1', 'B_2']
        assert len(cb) == 5
        assert cb[0] == (maintask, t1)
        assert cb[1] == (t1, t2)
        assert cb[2] == (t2, t1)
        assert cb[3] == (t1, t2)
        assert cb[4] == (t2, maintask)

    def test_bomb(self):
        try:
            1/0
        except:
            import sys
            b = stackless.bomb(*sys.exc_info())
        assert b.type is ZeroDivisionError
        assert str(b.value).startswith('integer division')
        assert b.traceback is not None

    def test_send_exception(self):
        def exp_sender(chan):
            chan.send_exception(Exception, 'test')

        def exp_recv(chan):
            try:
                val = chan.receive()
            except Exception as exp:
                assert exp.__class__ is Exception
                assert str(exp) == 'test'

        chan = stackless.channel()
        t1 = stackless.tasklet(exp_recv)(chan)
        t2 = stackless.tasklet(exp_sender)(chan)
        stackless.run()

    def test_send_sequence(self):
        res = []
        lst = [1,2,3,4,5,6,None]
        iterable = iter(lst)
        chan = stackless.channel()
        def f(chan):
            r = chan.receive()
            while r:
                res.append(r)
                r = chan.receive()

        t = stackless.tasklet(f)(chan)
        chan.send_sequence(iterable)
        assert res == [1,2,3,4,5,6]

    def test_getruncount(self):
        assert stackless.getruncount() == 1
        def with_schedule():
            assert stackless.getruncount() == 2

        t1 = stackless.tasklet(with_schedule)()
        assert stackless.getruncount() == 2
        stackless.schedule()
        def with_run():
            assert stackless.getruncount() == 1

        t2 = stackless.tasklet(with_run)()
        stackless.run()

    def test_schedule_return(self):
        def f():pass
        t1= stackless.tasklet(f)()
        r = stackless.schedule()
        assert r is stackless.getmain()
        t2 = stackless.tasklet(f)()
        r = stackless.schedule('test')
        assert r == 'test'

    def test_simple_pipe(self):
        def pipe(X_in, X_out):
            foo = X_in.receive()
            X_out.send(foo)

        X, Y = stackless.channel(), stackless.channel()
        t = stackless.tasklet(pipe)(X, Y)
        stackless.run()
        X.send(42)
        assert Y.receive() == 42

    def test_nested_pipe(self):
        dprint('tnp ==== 1')
        def pipe(X, Y):
            dprint('tnp_P ==== 1')
            foo = X.receive()
            dprint('tnp_P ==== 2')
            Y.send(foo)
            dprint('tnp_P ==== 3')

        def nest(X, Y):
            X2, Y2 = stackless.channel(), stackless.channel()
            t = stackless.tasklet(pipe)(X2, Y2)
            dprint('tnp_N ==== 1')
            X_Val = X.receive()
            dprint('tnp_N ==== 2')
            X2.send(X_Val)
            dprint('tnp_N ==== 3')
            Y2_Val = Y2.receive() 
            dprint('tnp_N ==== 4')
            Y.send(Y2_Val)
            dprint('tnp_N ==== 5')

        X, Y = stackless.channel(), stackless.channel()
        t1 = stackless.tasklet(nest)(X, Y)
        X.send(13)
        dprint('tnp ==== 2')
        res = Y.receive() 
        dprint('tnp ==== 3')
        assert res == 13
        if SHOW_STRANGE:
            raise Exception('force prints')

    def test_wait_two(self):
        """
        A tasklets/channels adaptation of the test_wait_two from the
        logic object space
        """
        def sleep(X, Y):
            dprint('twt_S ==== 1')
            value = X.receive()
            dprint('twt_S ==== 2')
            Y.send((X, value))
            dprint('twt_S ==== 3')

        def wait_two(X, Y, Ret_chan):
            Barrier = stackless.channel()
            stackless.tasklet(sleep)(X, Barrier)
            stackless.tasklet(sleep)(Y, Barrier)
            dprint('twt_W ==== 1')
            ret = Barrier.receive()
            dprint('twt_W ==== 2')
            if ret[0] == X:
                Ret_chan.send((1, ret[1]))
            else:
                Ret_chan.send((2, ret[1]))
            dprint('twt_W ==== 3')

        X = stackless.channel()
        Y = stackless.channel()
        Ret_chan = stackless.channel()

        stackless.tasklet(wait_two)(X, Y, Ret_chan)

        dprint('twt ==== 1')
        Y.send(42)

        dprint('twt ==== 2')
        X.send(42)
        dprint('twt ==== 3')
        value = Ret_chan.receive() 
        dprint('twt ==== 4')
        assert value == (2, 42)
        

    def test_schedule_return_value(self):

        def task(val):
            value = stackless.schedule(val)
            assert value == val

        stackless.tasklet(task)(10)
        stackless.tasklet(task)(5)

        stackless.run()

    def test_kill_tasklet_waiting_for_channel(self):
        # issue #2595
        c = stackless.channel()
        def sender():
            c.send(1)
        def receiver():
            v = c.receive()
        def killer(tl):
            tl.kill()
        def main():
            trk = stackless.tasklet(receiver)()
            stackless.schedule()
            killer(trk)
            stackless.schedule()
            stackless.tasklet(sender)()
            stackless.schedule()
            stackless.tasklet(receiver)()
            stackless.schedule()
        stackless.tasklet(main)()
        stackless.run()



import math, sys
from rpython.jit.metainterp.test.support import LLJitMixin
from rpython.rlib.rarithmetic import intmask, r_uint


class FloatTests:

    def test_simple(self):
        def f(a, b, c, d, e):
            return (((a + b) - c) * d) / e
        res = self.interp_operations(f, [41.5, 2.25, 17.5, 3.0, 2.5])
        assert res == 31.5

    def test_cast_bool_to_float(self):
        def f(a):
            return float(a == 12.0)
        res = self.interp_operations(f, [41.5])
        assert res == 0.0
        res = self.interp_operations(f, [12.0])
        assert res == 1.0

    def test_abs(self):
        def f(a):
            return abs(a)
        res = self.interp_operations(f, [-5.25])
        assert res == 5.25
        x = 281474976710656.31
        res = self.interp_operations(f, [x])
        assert res == x

    def test_neg(self):
        def f(a):
            return -a
        res = self.interp_operations(f, [-5.25])
        assert res == 5.25
        x = 281474976710656.31
        res = self.interp_operations(f, [x])
        assert res == -x

    def test_singlefloat(self):
        from rpython.rlib.rarithmetic import r_singlefloat
        def f(a):
            a = float(r_singlefloat(a))
            a *= 4.25
            return float(r_singlefloat(a))
        res = self.interp_operations(f, [-2.0], supports_singlefloats=True)
        assert res == -8.5

    def test_cast_float_to_int(self):
        def g(f):
            return int(f)
        res = self.interp_operations(g, [-12345.9])
        assert res == -12345

    def test_cast_float_to_uint(self):
        def g(f):
            return intmask(r_uint(f))
        res = self.interp_operations(g, [sys.maxint*2.0])
        assert res == intmask(long(sys.maxint*2.0))
        res = self.interp_operations(g, [-12345.9])
        assert res == -12345

    def test_cast_int_to_float(self):
        def g(i):
            return float(i)
        res = self.interp_operations(g, [-12345])
        assert type(res) is float and res == -12345.0

    def test_cast_int_to_float_constant(self):
        def h(i):
            if i < 10:
                i = 10
            return i
        def g(i):
            return float(h(i))
        res = self.interp_operations(g, [-12345])
        assert type(res) is float and res == 10.0

    def test_cast_uint_to_float(self):
        def g(i):
            return float(r_uint(i))
        res = self.interp_operations(g, [intmask(sys.maxint*2)])
        assert type(res) is float and res == float(sys.maxint*2)
        res = self.interp_operations(g, [-12345])
        assert type(res) is float and res == float(long(r_uint(-12345)))


class TestLLtype(FloatTests, LLJitMixin):
    pass



import pytest
from rpython.rlib.jit import JitDriver
from rpython.jit.metainterp.test.support import LLJitMixin
from rpython.jit.metainterp.blackhole import BlackholeInterpBuilder
from rpython.jit.metainterp.blackhole import BlackholeInterpreter
from rpython.jit.metainterp.blackhole import convert_and_run_from_pyjitpl
from rpython.jit.metainterp import history, pyjitpl, jitexc, resoperation
from rpython.jit.codewriter.assembler import JitCode
from rpython.rtyper.lltypesystem import lltype, llmemory
from rpython.rtyper.llinterp import LLException


class FakeCodeWriter:
    pass
class FakeAssembler:
    pass
class FakeCPU:
    def bh_call_i(self, func, args_i, args_r, args_f, calldescr):
        assert func == 321
        assert calldescr == "<calldescr>"
        if args_i[0] < 0:
            raise LLException("etype", "evalue")
        return args_i[0] * 2

def getblackholeinterp(insns, descrs=[]):
    cw = FakeCodeWriter()
    cw.cpu = FakeCPU()
    cw.assembler = FakeAssembler()
    cw.assembler.insns = insns
    cw.assembler.descrs = descrs
    builder = BlackholeInterpBuilder(cw)
    return builder.acquire_interp()

def test_simple():
    jitcode = JitCode("test")
    jitcode.setup("\x00\x00\x01\x02"
                  "\x01\x02",
                  [])
    blackholeinterp = getblackholeinterp({'int_add/ii>i': 0,
                                          'int_return/i': 1})
    blackholeinterp.setposition(jitcode, 0)
    blackholeinterp.setarg_i(0, 40)
    blackholeinterp.setarg_i(1, 2)
    blackholeinterp.run()
    assert blackholeinterp._final_result_anytype() == 42

def test_simple_const():
    jitcode = JitCode("test")
    jitcode.setup("\x00\x30\x01\x02"
                  "\x01\x02",
                  [])
    blackholeinterp = getblackholeinterp({'int_sub/ci>i': 0,
                                          'int_return/i': 1})
    blackholeinterp.setposition(jitcode, 0)
    blackholeinterp.setarg_i(1, 6)
    blackholeinterp.run()
    assert blackholeinterp._final_result_anytype() == 42

def test_simple_bigconst():
    jitcode = JitCode("test")
    jitcode.setup("\x00\x05\x01\x02"
                  "\x01\x02",
                  [666, 666, 10042, 666],
                  num_regs_i=3)
    blackholeinterp = getblackholeinterp({'int_sub/ii>i': 0,
                                          'int_return/i': 1})
    blackholeinterp.setposition(jitcode, 0)
    blackholeinterp.setarg_i(1, 10000)
    blackholeinterp.run()
    assert blackholeinterp._final_result_anytype() == 42

def test_simple_loop():
    jitcode = JitCode("test")
    jitcode.setup("\x00\x16\x02\x10\x00"  # L1: goto_if_not_int_gt %i0, 2, L2
                  "\x01\x17\x16\x17"      #     int_add %i1, %i0, %i1
                  "\x02\x16\x01\x16"      #     int_sub %i0, $1, %i0
                  "\x03\x00\x00"          #     goto L1
                  "\x04\x17",             # L2: int_return %i1
                  [])
    blackholeinterp = getblackholeinterp({'goto_if_not_int_gt/icL': 0,
                                          'int_add/ii>i': 1,
                                          'int_sub/ic>i': 2,
                                          'goto/L': 3,
                                          'int_return/i': 4})
    blackholeinterp.setposition(jitcode, 0)
    blackholeinterp.setarg_i(0x16, 6)    # %i0
    blackholeinterp.setarg_i(0x17, 100)  # %i1
    blackholeinterp.run()
    assert blackholeinterp._final_result_anytype() == 100+6+5+4+3

def test_simple_exception():
    jitcode = JitCode("test")
    jitcode.setup(    # residual_call_ir_i $<* fn g>, I[%i9], R[], <Descr>  %i8
                  "\x01\xFF\x01\x09\x00\x00\x00\x08"
                  "\x00\x0D\x00"          #     catch_exception L1
                  "\x02\x08"              #     int_return %i8
                  "\x03\x2A",             # L1: int_return $42
                  [321])   # <-- address of the function g
    blackholeinterp = getblackholeinterp({'catch_exception/L': 0,
                                          'residual_call_ir_i/iIRd>i': 1,
                                          'int_return/i': 2,
                                          'int_return/c': 3},
                                         ["<calldescr>"])
    #
    blackholeinterp.setposition(jitcode, 0)
    blackholeinterp.setarg_i(0x9, 100)
    blackholeinterp.run()
    assert blackholeinterp._final_result_anytype() == 200
    #
    blackholeinterp.setposition(jitcode, 0)
    blackholeinterp.setarg_i(0x9, -100)
    blackholeinterp.run()
    assert blackholeinterp._final_result_anytype() == 42

def test_convert_and_run_from_pyjitpl():
    class MyMIFrame:
        jitcode = JitCode("test")
        jitcode.setup("\xFF"               # illegal instruction
                      "\x00\x00\x01\x02"   # int_add/ii>i
                      "\x01\x02",          # int_return/i
                      [],
                      num_regs_i=3, num_regs_r=0, num_regs_f=0)
        jitcode.jitdriver_sd = "foo" # not none
        pc = 1
        registers_i = [resoperation.InputArgInt(40), history.ConstInt(2), None]
    class MyMetaInterp:
        class staticdata:
            result_type = 'int'
            class profiler:
                @staticmethod
                def start_blackhole(): pass
                @staticmethod
                def end_blackhole(): pass
        last_exc_value = None
        framestack = [MyMIFrame()]
    MyMetaInterp.staticdata.blackholeinterpbuilder = getblackholeinterp(
        {'int_add/ii>i': 0, 'int_return/i': 1}).builder
    MyMetaInterp.staticdata.blackholeinterpbuilder.metainterp_sd = \
        MyMetaInterp.staticdata
    #
    d = pytest.raises(jitexc.DoneWithThisFrameInt,
                       convert_and_run_from_pyjitpl, MyMetaInterp())
    assert d.value.result == 42


class TestBlackhole(LLJitMixin):

    @pytest.mark.xfail
    def test_blackholeinterp_cache_basic(self):
        class FakeJitcode:
            def num_regs_r(self):
                return 0
        interp1 = getblackholeinterp({})
        interp1.jitcode = FakeJitcode()
        builder = interp1.builder
        interp2 = builder.acquire_interp()
        builder.release_interp(interp1)
        interp3 = builder.acquire_interp()
        assert builder.num_interpreters == 2

    @pytest.mark.xfail
    def test_blackholeinterp_cache_normal(self):
        myjitdriver = JitDriver(greens = [], reds = ['x', 'y'])
        def choices(x):
            if x == 0:       # <- this is the test that eventually succeeds,
                return 0     #    requiring a blackhole interp in a call stack
            return 34871     #    of two functions (hence num_interpreters==2)
        def f(x):
            y = 0
            cont = 1
            while cont:
                myjitdriver.can_enter_jit(x=x, y=y)
                myjitdriver.jit_merge_point(x=x, y=y)
                cont = choices(x)
                y += cont
                x -= 1
            return y
        #
        seen = []
        def my_copy_constants(self, *args):
            seen.append(1)
            return org_copy_constants(self, *args)
        org_copy_constants = BlackholeInterpreter.copy_constants
        BlackholeInterpreter.copy_constants = my_copy_constants
        try:
            res = self.meta_interp(f, [7], repeat=7)
        finally:
            BlackholeInterpreter.copy_constants = org_copy_constants
        #
        assert res == sum([choices(x) for x in range(1, 8)])
        builder = pyjitpl._warmrunnerdesc.metainterp_sd.blackholeinterpbuilder
        assert builder.num_interpreters == 2
        assert len(seen) == 2 * 3

    @pytest.mark.xfail
    def test_blackholeinterp_cache_exc(self):
        myjitdriver = JitDriver(greens = [], reds = ['x', 'y'])
        class FooError(Exception):
            def __init__(self, num):
                self.num = num
        def choices(x):
            if x == 0:
                raise FooError(0)
            raise FooError(34871)
        def f(x):
            y = 0
            while True:
                myjitdriver.can_enter_jit(x=x, y=y)
                myjitdriver.jit_merge_point(x=x, y=y)
                try:
                    choices(x)
                except FooError as e:
                    if e.num == 0:
                        break
                    y += e.num
                x -= 1
            return y
        res = self.meta_interp(f, [7], repeat=7)
        assert res == sum([pytest.raises(FooError, choices, x).value.num
                           for x in range(1, 8)])
        builder = pyjitpl._warmrunnerdesc.metainterp_sd.blackholeinterpbuilder
        assert builder.num_interpreters == 2

def test_bad_shift():
    pytest.raises(ValueError, BlackholeInterpreter.bhimpl_int_lshift.im_func, 7, 100)
    pytest.raises(ValueError, BlackholeInterpreter.bhimpl_int_rshift.im_func, 7, 100)
    pytest.raises(ValueError, BlackholeInterpreter.bhimpl_uint_rshift.im_func, 7, 100)
    pytest.raises(ValueError, BlackholeInterpreter.bhimpl_int_lshift.im_func, 7, -1)
    pytest.raises(ValueError, BlackholeInterpreter.bhimpl_int_rshift.im_func, 7, -1)
    pytest.raises(ValueError, BlackholeInterpreter.bhimpl_uint_rshift.im_func, 7, -1)

    assert BlackholeInterpreter.bhimpl_int_lshift.im_func(100, 3) == 100<<3
    assert BlackholeInterpreter.bhimpl_int_rshift.im_func(100, 3) == 100>>3
    assert BlackholeInterpreter.bhimpl_uint_rshift.im_func(100, 3) == 100>>3

def test_debug_fatalerror():
    from rpython.rtyper.lltypesystem import lltype, llmemory, rstr
    from rpython.rtyper.llinterp import LLFatalError
    msg = rstr.mallocstr(1)
    msg.chars[0] = "!"
    msg = lltype.cast_opaque_ptr(llmemory.GCREF, msg)
    e = pytest.raises(LLFatalError,
                       BlackholeInterpreter.bhimpl_debug_fatalerror.im_func,
                       msg)
    assert str(e.value) == '!'



import os
from rpython.flowspace.model import Constant

SPECIAL_CASES = {}

def register_flow_sc(func):
    """Decorator triggering special-case handling of ``func``.

    When the flow graph builder sees ``func``, it calls the decorated function
    with ``decorated_func(ctx, *args_w)``, where ``args_w`` is a sequence of
    flow objects (Constants or Variables).
    """
    def decorate(sc_func):
        SPECIAL_CASES[func] = sc_func
    return decorate

def redirect_function(srcfunc, dstfuncname):
    @register_flow_sc(srcfunc)
    def sc_redirected_function(ctx, *args_w):
        components = dstfuncname.split('.')
        obj = __import__('.'.join(components[:-1]))
        for name in components[1:]:
            obj = getattr(obj, name)
        return ctx.appcall(obj, *args_w)


@register_flow_sc(__import__)
def sc_import(ctx, *args_w):
    assert all(isinstance(arg, Constant) for arg in args_w)
    args = [arg.value for arg in args_w]
    return ctx.import_name(*args)

@register_flow_sc(locals)
def sc_locals(_, *args):
    raise Exception(
        "A function calling locals() is not RPython.  "
        "Note that if you're translating code outside the PyPy "
        "repository, a likely cause is that py.test's --assert=rewrite "
        "mode is getting in the way.  You should copy the file "
        "pytest.ini from the root of the PyPy repository into your "
        "own project.")

@register_flow_sc(getattr)
def sc_getattr(ctx, w_obj, w_index, w_default=None):
    if w_default is not None:
        return ctx.appcall(getattr, w_obj, w_index, w_default)
    else:
        from rpython.flowspace.operation import op
        return op.getattr(w_obj, w_index).eval(ctx)

# _________________________________________________________________________

redirect_function(open,       'rpython.rlib.rfile.create_file')
redirect_function(os.fdopen,  'rpython.rlib.rfile.create_fdopen_rfile')
redirect_function(os.tmpfile, 'rpython.rlib.rfile.create_temp_rfile')

# on top of PyPy only: 'os.remove != os.unlink'
# (on CPython they are '==', but not identical either)
redirect_function(os.remove,  'os.unlink')

redirect_function(os.path.isdir,   'rpython.rlib.rpath.risdir')
redirect_function(os.path.isabs,   'rpython.rlib.rpath.risabs')
redirect_function(os.path.normpath,'rpython.rlib.rpath.rnormpath')
redirect_function(os.path.abspath, 'rpython.rlib.rpath.rabspath')
redirect_function(os.path.join,    'rpython.rlib.rpath.rjoin')
if hasattr(os.path, 'splitdrive'):
    redirect_function(os.path.splitdrive, 'rpython.rlib.rpath.rsplitdrive')

# _________________________________________________________________________
# a simplified version of the basic printing routines, for RPython programs
class StdOutBuffer:
    linebuf = []
    final = True
stdoutbuffer = StdOutBuffer()

def rpython_print_item(s):
    buf = stdoutbuffer.linebuf
    if not stdoutbuffer.final:
        buf.append(' ')
    for c in s:
        buf.append(c)
    stdoutbuffer.final = False
rpython_print_item._annenforceargs_ = (str,)

def rpython_print_end(s):
    buf = stdoutbuffer.linebuf
    stdoutbuffer.final = True
    for c in s:
        buf.append(c)
        if c == '\n':
            w = ''.join(buf)
            del buf[:]
            os.write(1, w)

def rpython_print_newline():
    rpython_print_end('\n')




class AppTestJitNotInTrace(object):
    spaceconfig = dict(usemodules=('pypyjit',))

    def test_not_from_assembler(self):
        import pypyjit
        @pypyjit.not_from_assembler
        def f(x, y):
            return 42
        r = f(3, 4)
        assert r is f

    def test_not_from_assembler_exception(self):
        import pypyjit
        @pypyjit.not_from_assembler
        def f(x, y):
            raise ValueError(y, x)
        e = raises(ValueError, f, 3, 4)
        assert e.value.args == (4, 3)



import glob, os, sys, subprocess

USES_PYTHON_CAPI = set(('pythonizables',))

fn = sys.argv[1]

if fn == 'all':
    all_headers = glob.glob('*.h')
    for header in all_headers:
        res = os.system(" ".join(['python', sys.argv[0], header[:-2]]+sys.argv[2:]))
        if res != 0:
            sys.exit(res)
    sys.exit(0)
else:
    if fn[-4:] == '.cxx': fn = fn[:-4]
    elif fn[-2:] == '.h': fn = fn[:-2]
    if not os.path.exists(fn+'.h'):
        print("file %s.h does not exist" % (fn,))
        sys.exit(1)

uses_python_capi = False
if fn in USES_PYTHON_CAPI:
    uses_python_capi = True

if os.path.exists(fn+'Dict.dll'):
    dct_time = os.stat(fn+'Dict.dll').st_mtime
    if not '-f' in sys.argv:
        mustbuild = False
        for ext in ['.h', '.cxx', '.xml']:
            if os.stat(fn+ext).st_mtime > dct_time:
                mustbuild = True
                break
        if not mustbuild:
            sys.exit(0)

 # cleanup
    for fg in set(glob.glob(fn+"_rflx*") + glob.glob(fn+"Dict*") + \
                  glob.glob("*.obj") + glob.glob(fn+"Linkdef.h")):
        os.remove(fg)

def _get_config_exec():
        return [sys.executable, '-m', 'cppyy_backend._cling_config']

def get_config(what):
    config_exec_args = _get_config_exec()
    config_exec_args.append('--'+what)
    cli_arg = subprocess.check_output(config_exec_args)
    return cli_arg.decode("utf-8").strip()

def get_python_include_dir():
    incdir = subprocess.check_output([sys.executable, '-c', "import sysconfig; print(sysconfig.get_path('include'))"])
    return incdir.decode("utf-8").strip()

def get_python_lib_dir():
    libdir = subprocess.check_output([sys.executable, '-c', "import sysconfig; print(sysconfig.get_path('stdlib'))"])
    return os.path.join(os.path.dirname(libdir.decode("utf-8").strip()), 'libs')

# genreflex option
#DICTIONARY_CMD = "genreflex {fn}.h --selection={fn}.xml --rootmap={fn}Dict.rootmap --rootmap-lib={fn}Dict.dll".format(fn=fn)

with open(fn+'Linkdef.h', 'w') as linkdef:
    linkdef.write("#ifdef __CLING__\n\n")
    linkdef.write("#pragma link C++ defined_in %s.h;\n" % fn)
    linkdef.write("\n#endif")

DICTIONARY_CMD = "python -m cppyy_backend._rootcling -f {fn}_rflx.cxx -rmf {fn}Dict.rootmap -rml {fn}Dict.dll {fn}.h {fn}Linkdef.h".format(fn=fn)
if os.system(DICTIONARY_CMD):
    sys.exit(1)

import platform
if '64' in platform.architecture()[0]:
    PLATFORMFLAG = '-D_AMD64_'
    MACHINETYPE  = 'X64'
else:
    PLATFORMFLAG = '-D_X86_'
    MACHINETYPE  = 'IX86'

cppflags = get_config('cppflags')
if uses_python_capi:
    cppflags += ' -I"' + get_python_include_dir() + '"'
BUILDOBJ_CMD_PART = "cl -O2 -nologo -TP -c -nologo " + cppflags + " -FIsehmap.h -Zc:__cplusplus -MD -GR -D_WINDOWS -DWIN32 " + PLATFORMFLAG + " -EHsc- -W3 -wd4141 -wd4291 -wd4244 -wd4049 -D_XKEYCHECK_H -D_LIBCPP_HAS_NO_PRAGMA_SYSTEM_HEADER -DNOMINMAX -D_CRT_SECURE_NO_WARNINGS {fn}.cxx -Fo{fn}.obj"
BUILDOBJ_CMD = BUILDOBJ_CMD_PART.format(fn=fn)
if os.system(BUILDOBJ_CMD):
    sys.exit(1)
BUILDOBJ_CMD = BUILDOBJ_CMD_PART.format(fn=fn+'_rflx')
if os.system(BUILDOBJ_CMD):
    sys.exit(1)

import cppyy_backend
CREATEDEF_CMD = "python bindexplib.py {fn} {fn}Dict".format(fn=fn)
if os.system(CREATEDEF_CMD):
    sys.exit(1)

ldflags = ''
if uses_python_capi:
    ldflags = ' /LIBPATH:"' + get_python_lib_dir() + '" '
CREATELIB_CMD = ("lib -nologo -MACHINE:" + MACHINETYPE + " -out:{fn}Dict.lib {fn}.obj {fn}_rflx.obj -def:{fn}Dict.def " + ldflags).format(fn=fn)
if os.system(CREATELIB_CMD):
    sys.exit(1)

ldflags += get_config('ldflags')
LINKDLL_CMD = ("link -nologo {fn}.obj {fn}_rflx.obj -DLL -out:{fn}Dict.dll {fn}Dict.exp " + ldflags).format(fn=fn)
if os.system(LINKDLL_CMD):
    sys.exit(1)

# cleanup
for fg in set(glob.glob(fn+"_rflx.cxx*") + glob.glob("*.obj") + glob.glob(fn+"Linkdef.h")):
    os.remove(fg)



from pypy.objspace.fake.checkmodule import checkmodule

def test_checkmodule():
    # we need to ignore GcCollectStepStats, else checkmodule fails. I think
    # this happens because W_GcCollectStepStats.__init__ is only called from
    # GcCollectStepHookAction.perform() and the fake objspace doesn't know
    # about those: so, perform() is never annotated and the annotator thinks
    # W_GcCollectStepStats has no attributes
    checkmodule('gc', ignore=['GcCollectStepStats'])



import py
import random, sys
from rpython.rlib.rope import *

def make_random_string(operations=10, slicing=True, print_seed=True,
                       unicode=False):
    seed = random.randrange(10000)
    if print_seed:
        print seed
    random.seed(seed)
    st = "abc"
    curr = LiteralStringNode(st)
    if slicing:
        choice = [0, 1, 2]
    else:
        choice = [0, 1]
    for i in range(operations):
        if not unicode:
            a = (chr(random.randrange(ord('a'), ord('z') + 1)) *
                    random.randrange(500))
            node = LiteralStringNode(a)
        else:
            a = (unichr(random.randrange(sys.maxunicode)) *
                        random.randrange(500))
            node = LiteralUnicodeNode(a)
        c = random.choice(choice)
        if c == 0:
            curr = curr + node
            st = st + a
        elif c == 1:
            curr = node + curr
            st = a + st
        else:
            if len(st) < 10:
                continue
            start = random.randrange(len(st) // 3)
            stop = random.randrange(len(st) // 3 * 2, len(st))
            curr = getslice_one(curr, start, stop)
            st = st[start: stop]
    return curr, st


def test_add():
    s = (LiteralStringNode("a" * 32) + LiteralStringNode("bc" * 32) +
         LiteralStringNode("d" * 32) + LiteralStringNode("ef" * 32) +
         LiteralStringNode(""))
    assert s.depth() == 3
    assert s.flatten_string() == "".join([c * 32 for c in "a", "bc", "d", "ef"])
    s = s.rebalance()
    assert s.flatten_string() == "".join([c * 32 for c in "a", "bc", "d", "ef"])

def test_dont_rebalance_again():
    s = (LiteralStringNode("a" * 32) + LiteralStringNode("b" * 32) +
         LiteralStringNode("d" * 32) + LiteralStringNode("e" * 32) +
         LiteralStringNode(""))
    assert s.depth() == 3
    assert s.flatten_string() == "".join([c * 32 for c in "abde"])
    s = s.rebalance()
    assert s.check_balanced()
    assert s.balanced
    assert s.flatten_string() == "".join([c * 32 for c in "abde"])

def test_random_addition_test():
    seed = random.randrange(10000)
    print seed # 4443
    st = "abc"
    curr = LiteralStringNode(st)
    for i in range(1000):
        a = (chr(random.randrange(ord('a'), ord('z') + 1)) *
                random.randrange(100))
        if random.choice([0, 1]):
            curr = curr + LiteralStringNode(a)
            st = st + a
        else:
            curr = LiteralStringNode(a) + curr
            st = a + st
        assert curr.flatten_string() == st
    curr = curr.rebalance()
    assert curr.flatten_string() == st

def test_getitem():
    result = "".join([c * 32 for c in "a", "bc", "d", "ef"])
    s1 = (LiteralStringNode("a" * 32) + LiteralStringNode("bc" * 32) +
          LiteralStringNode("d" * 32) + LiteralStringNode("ef" * 32) +
          LiteralStringNode(""))
    s2 = s1.rebalance()
    for i in range(len(result)):
        for s in [s1, s2]:
            assert s.getchar(i) == result[i]
            assert s.getint(i) == ord(result[i])

def test_getitem_unicode():
    s1, result = make_random_string(200, unicode=True)
    s2 = s1.rebalance()
    for i in range(len(result)):
        for s in [s1, s2]:
            assert s.getunichar(i) == result[i]
            assert s.getint(i) == ord(result[i])

def test_getslice():
    result = "".join([c * 32 for c in "a", "bc", "d", "ef"])
    s1 = (LiteralStringNode("a" * 32) + LiteralStringNode("bc" * 32) +
          LiteralStringNode("d" * 32) + LiteralStringNode("ef" * 32) +
          LiteralStringNode(""))
    s2 = s1.rebalance()
    for s in [s1, s2]:
        for start in range(0, len(result)):
            for stop in range(start, len(result)):
                assert getslice_one(s, start, stop).flatten_string() == result[start:stop]

def test_getslice_bug():
    s1 = LiteralStringNode("/home/arigo/svn/pypy/branch/rope-branch/pypy/bin")
    s2 = LiteralStringNode("/pypy")
    s = s1 + s2
    r = getslice_one(s, 1, 5)
    assert r.flatten_string() == "home"


def test_getslice_step():
    s1 = (LiteralStringNode("abcde") + LiteralStringNode("fghijklm") +
          LiteralStringNode("nopqrstu") + LiteralStringNode("vwxyz") + 
          LiteralStringNode("zyxwvut") + LiteralStringNode("srqpomnlk"))
    s2 = s1.rebalance()
    result = s1.flatten_string()
    assert s2.flatten_string() == result
    for s in [s1, s2]:
        for start in range(0, len(result)):
            for stop in range(start, len(result)):
                for step in range(1, stop - start):
                    assert getslice(s, start, stop, step).flatten_string() == result[start:stop:step]

def test_getslice_step_unicode():
    s1 = (LiteralUnicodeNode(u"\uaaaa") +
          LiteralUnicodeNode(u"\ubbbb" * 5) +
          LiteralUnicodeNode(u"\uaaaa\ubbbb\u1000\u2000") +
          LiteralUnicodeNode(u"vwxyz") + 
          LiteralUnicodeNode(u"zyxwvu\u1234" * 2) +
          LiteralUnicodeNode(u"12355"))
    s2 = s1.rebalance()
    result = s1.flatten_unicode()
    assert s2.flatten_unicode() == result
    for s in [s1, s2]:
        for start in range(0, len(result)):
            for stop in range(start, len(result)):
                for step in range(1, stop - start):
                    assert getslice(s, start, stop, step).flatten_unicode() == result[start:stop:step]

def test_random_addition_and_slicing():
    seed = random.randrange(10000)
    print seed
    random.seed(seed)
    st = "abc"
    curr = LiteralStringNode(st)
    last = None
    all = []
    for i in range(1000):
        a = (chr(random.randrange(ord('a'), ord('z') + 1)) *
                random.randrange(500))
        last = curr
        all.append(curr)
        c = random.choice([0, 1, 2])
        if c == 0:
            curr = curr + LiteralStringNode(a)
            st = st + a
        elif c == 1:
            curr = LiteralStringNode(a) + curr
            st = a + st
        else:
            if len(st) < 10:
                continue
            # get a significant portion of the string
            #import pdb; pdb.set_trace()
            start = random.randrange(len(st) // 3)
            stop = random.randrange(len(st) // 3 * 2, len(st))
            curr = getslice_one(curr, start, stop)
            st = st[start: stop]
        assert curr.flatten_string() == st
    curr = curr.rebalance()
    assert curr.flatten_string() == st

def test_iteration():
    rope, real_st = make_random_string(200)
    iter = ItemIterator(rope)
    for c in real_st:
        c2 = iter.nextchar()
        assert c2 == c
    py.test.raises(StopIteration, iter.nextchar)
    iter = ItemIterator(rope)
    for c in real_st:
        c2 = iter.nextunichar()
        assert c2 == c
    py.test.raises(StopIteration, iter.nextchar)
    iter = ItemIterator(rope)
    for c in real_st:
        c2 = iter.nextint()
        assert c2 == ord(c)
    py.test.raises(StopIteration, iter.nextchar)

def test_iteration_startpos():
    rope, real_st = make_random_string(200)
    for i in range(0, len(real_st), len(real_st) // 20):
        iter = ItemIterator(rope, i)
        x = i
        for c in real_st[i:]:
            x += 1
            c2 = iter.nextchar()
            assert c2 == c
        py.test.raises(StopIteration, iter.nextchar)

def test_iteration_unicode():
    rope, real_st = make_random_string(200, unicode=True)
    iter = ItemIterator(rope)
    for c in real_st:
        c2 = iter.nextunichar()
        assert c2 == c
    py.test.raises(StopIteration, iter.nextchar)
    iter = ItemIterator(rope)
    for c in real_st:
        c2 = iter.nextint()
        assert c2 == ord(c)
    py.test.raises(StopIteration, iter.nextchar)

def test_reverse_iteration():
    rope, real_st = make_random_string(200)
    iter = ReverseItemIterator(rope)
    for c in reversed(real_st):
        c2 = iter.nextchar()
        assert c2 == c
    py.test.raises(StopIteration, iter.nextchar)
    iter = ReverseItemIterator(rope)
    for c in py.builtin.reversed(real_st):
        c2 = iter.nextint()
        assert c2 == ord(c)
    py.test.raises(StopIteration, iter.nextchar)

def test_reverse_iteration_unicode():
    rope, real_st = make_random_string(200, unicode=True)
    iter = ReverseItemIterator(rope)
    for c in reversed(real_st):
        c2 = iter.nextunichar()
        assert c2 == c
    py.test.raises(StopIteration, iter.nextchar)
    iter = ReverseItemIterator(rope)
    for c in reversed(real_st):
        c2 = iter.nextint()
        assert c2 == ord(c)
    py.test.raises(StopIteration, iter.nextchar)

def test_multiply():
    strs = [(LiteralStringNode("a"), "a"), (LiteralStringNode("abc"), "abc"),
            make_random_string(500)]
    times = range(100)
    for i in range(9, 30):
        times.append(i ** 2 - 1)
        times.append(i ** 2)
        times.append(i ** 2 + 1)
        times.append(i ** 2 + 2)
    for r, st in strs:
        for i in times:
            r2 = multiply(r, i)
            assert r2.flatten_string() == st * i

def test_multiply_unicode():
    strs = [(LiteralUnicodeNode(u"\uaaaa"), u"\uaaaa"),
            (LiteralUnicodeNode(u"\uaaaa\ubbbb\ucccc"), u"\uaaaa\ubbbb\ucccc"),
            make_random_string(500)]
    times = range(100)
    for i in range(9, 30):
        times.append(i ** 2 - 1)
        times.append(i ** 2)
        times.append(i ** 2 + 1)
        times.append(i ** 2 + 2)
    for r, st in strs:
        for i in times:
            r2 = multiply(r, i)
            assert r2.flatten_unicode() == st * i

def test_join():
    seps = [(LiteralStringNode("a"), "a"), (LiteralStringNode("abc"), "abc"),
            (LiteralStringNode("d"), "d"), (LiteralStringNode(""), "")]
    l, strs = zip(*[(LiteralStringNode("x"), "x"),
                    (LiteralStringNode("xyz"), "xyz"),
                    (LiteralStringNode("w"), "w")])
    l = list(l)
    for s, st in seps:
        node = join(s, l)
        result1 = node.flatten_string()
        result2 = st.join(strs)
        for i in range(node.length()):
            assert result1[i] == result2[i]

    strings = ['', '<',
               '/home/arigo/svn/pypy/branch/rope-branch/py/code/source.py',
               ':', '213', '>']
    l = [LiteralStringNode(s) for s in strings]
    node = join(LiteralStringNode(""), l)
    assert node.flatten_string() == ''.join(strings)

def test_join_random():
    l, strs = zip(*[make_random_string(10 * i) for i in range(1, 5)])
    l = list(l)
    seps = [(LiteralStringNode("a"), "a"), (LiteralStringNode("abc"), "abc"),
            make_random_string(500)]
    for s, st in seps:
        node = join(s, l)
        result1 = node.flatten_string()
        result2 = st.join(strs)
        for i in range(node.length()):
            assert result1[i] == result2[i]

def test_join_random_unicode():
    l, strs = zip(*[make_random_string(10 * i, unicode=True) for i in range(1, 5)])
    l = list(l)
    seps = [(LiteralStringNode("a"), "a"), (LiteralStringNode("abc"), "abc"),
            make_random_string(500)]
    for s, st in seps:
        node = join(s, l)
        result1 = node.flatten_unicode()
        result2 = st.join(strs)
        for i in range(node.length()):
            assert result1[i] == result2[i]

def test_seekbackward():
    rope = BinaryConcatNode(BinaryConcatNode(LiteralStringNode("abc"),
                                             LiteralStringNode("def")),
                            LiteralStringNode("ghi"))
    iter = SeekableItemIterator(rope)
    for c in "abcdefgh":
        c2 = iter.nextchar()
        assert c2 == c
    for i in range(7):
        iter.seekback(i)
        for c in "abcdefghi"[-1-i:-1]:
            c2 = iter.nextchar()
            assert c2 == c
    c2 = iter.nextchar()
    assert c2 == "i"
    py.test.raises(StopIteration, iter.nextchar)

def test_fringe_iterator():
    ABC = LiteralStringNode("abc")
    DEF = LiteralStringNode("def")
    GHI = LiteralStringNode("ghi")
    rope = BinaryConcatNode(BinaryConcatNode(ABC, DEF), GHI)
    iter = FringeIterator(rope)
    n = iter.next()
    assert n is ABC
    n = iter.next()
    assert n is DEF
    n = iter.next()
    assert n is GHI
    py.test.raises(StopIteration, iter.next)
    iter = FringeIterator(rope)

def test_fringe_iterator_seekforward():
    ABC = LiteralStringNode("abc")
    DEF = LiteralStringNode("def")
    GHI = LiteralStringNode("ghi")
    rope = BinaryConcatNode(BinaryConcatNode(ABC, DEF), GHI)
    iter = FringeIterator(rope)
    n = iter.next()
    assert n is ABC
    i = iter._seekforward(5)
    assert i == 2
    n = iter.next()
    assert n is GHI
    py.test.raises(StopIteration, iter.next)
    iter = FringeIterator(rope)
    i = iter._seekforward(7)
    assert i == 1
    n = iter.next()
    assert n is GHI


def test_seekforward():
    rope = BinaryConcatNode(BinaryConcatNode(LiteralStringNode("abc"),
                                             LiteralStringNode("def")),
                            LiteralStringNode("ghi"))
    rope = rope + rope
    result = rope.flatten_string()
    for j in range(len(result) - 1):
        for i in range(len(result) - 1 - j):
            iter = SeekableItemIterator(rope)
            for c in result[:j]:
                c2 = iter.nextchar()
                assert c2 == c
            iter.seekforward(i)
            for c in result[i + j:]:
                c2 = iter.nextchar()
                assert c2 == c
            py.test.raises(StopIteration, iter.nextchar)

def test_iterbackward():
    rope = BinaryConcatNode(BinaryConcatNode(LiteralStringNode("abc"),
                                             LiteralStringNode("def")),
                            LiteralStringNode("ghi"))
    iter = SeekableItemIterator(rope)
    iter.seekforward(8)
    for c in "abcdefghi"[::-1]:
        c2 = iter.lastchar()
        assert c2 == c
    py.test.raises(StopIteration, iter.lastchar)

def test_find_int():
    rope, st = make_random_string()
    rope = getslice_one(rope, 10, 100)
    st = st[10:100]
    for i in range(len(st)):
        for j in range(i + 1, len(st)):
            c = st[i:j][(j - i) // 2]
            pos = find_int(rope, ord(c), i, j)
            assert pos == st.find(c, i, j)

def test_find_int_bugs():
    r = find_int(LiteralStringNode("ascii"), ord(" "), 0, 5)
    assert r == -1
    r = find_int(LiteralStringNode("a"), ord("a"))
    assert r == 0


def test_restart_positions():
    restart = construct_restart_positions_node(
        BinaryConcatNode(LiteralStringNode("aba"), LiteralStringNode("bcabab")))
    assert restart == [0, 0, 1, 2, 0, 1, 2, 3, 4]
    restart = construct_restart_positions_node(
        BinaryConcatNode(LiteralStringNode("aba"), LiteralStringNode("bcababb")))
    assert restart == [0, 0, 1, 2, 0, 1, 2, 3, 4, 0]
    restart = construct_restart_positions_node(LiteralStringNode("ababb"))
    assert restart == [0, 0, 1, 2, 0]


def test_find():
    node = BinaryConcatNode(LiteralStringNode("aba"),
                            LiteralStringNode("bcabab"))
    pos = find(node, LiteralStringNode("abc"), 0, node.length())
    assert pos == 2
    node = BinaryConcatNode(LiteralStringNode("btffp"),
                            LiteralStringNode("bacbb"))
    pos = find(node, LiteralStringNode("a"), 0, node.length())
    assert pos == 6
    pos = find(node, LiteralStringNode("aaa"), 0, 2)
    assert pos == -1
    pos = find(node, LiteralStringNode("btf"), 0, 3)
    assert pos == 0

def test_find_random():
    py.test.skip("fix me!")
    rope, st = make_random_string(unicode=True)
    rope = getslice_one(rope, 10, 10000)
    st = st[10:10000]
    for i in range(1000):
        searchlength = random.randrange(2, min(len(st) - 1, 1001))
        start = random.randrange(len(st) - searchlength)
        searchstart = random.randrange(len(st))
        searchstop = random.randrange(searchstart, len(st))
        p = st[start:start+searchlength]
        rp = getslice_one(rope, start, start + searchlength)
        pos = find(rope, rp, searchstart, searchstop)
        assert pos == st.find(p, searchstart, searchstop)

def test_find_unicode():
    node = BinaryConcatNode(LiteralUnicodeNode(u"\uaaaa\ubbbb\uaaaa"),
                            LiteralUnicodeNode(u"\ubbbb\ucccc\uaaaa\ubbbb\uaaaa\ubbbb"))
    pos = find(node, LiteralUnicodeNode(u"\uaaaa\ubbbb\ucccc"), 0, node.length())
    assert pos == 2
    node = BinaryConcatNode(LiteralUnicodeNode(u"btffp"),
                            LiteralUnicodeNode(u"b\uaaaacbb"))
    pos = find(node, LiteralUnicodeNode(u"\uaaaa"), 0, node.length())
    assert pos == 6

def test_fromcharlist():
    for i in range(0, 100, 10):
        chars = ["a"] * 50 + ["b"] * i
        node = rope_from_charlist(chars)
        assert node.flatten_string() == "a" * 50  + "b" * i
    assert rope_from_charlist([]).flatten_string() == ""

def test_find_iterator():
    for searchstring in ["abc", "a", "", "x", "xyz", "abababcabcabb"]:
        node = join(LiteralStringNode(searchstring),
                    [LiteralStringNode("cde" * i) for i in range(1, 10)])
        #node.view()
        iter = FindIterator(node, LiteralStringNode(searchstring))
        s = node.flatten_string()
        assert s == searchstring.join(["cde" * i for i in range(1, 10)])
        start = 0
        while 1:
            r2 = s.find(searchstring, start)
            try:
                r1 = iter.next()
            except StopIteration:
                assert r2 == -1
                break
            assert r1 == r2
            start = r2 + max(len(searchstring), 1)


def test_find_iterator_unicode():
    if sys.version_info < (2, 5):
        py.test.skip("bug in unicode.find that was fixed in 2.5")
    for searchstring in [
        u"\uAAAA\uBBBB\uCCCC", u"\uAAAA", u"", u"\u6666",
        u"\u6666\u7777\u8888",
        u"\uAAAA\uBBBB\uAAAA\uBBBB\uAAAA\uBBBB\uCCCC\uAAAA\uBBBB\uCCCC\uAAAA\uBBBB\uBBBB"]:
        node = join(LiteralUnicodeNode(searchstring),
                    [LiteralUnicodeNode(u"\ucccc\udddd" * i)
                        for i in range(1, 10)])
        iter = FindIterator(node, LiteralUnicodeNode(searchstring))
        s = node.flatten_unicode()
        assert s == searchstring.join([u"\ucccc\udddd" * i for i in range(1, 10)])
        start = 0
        while 1:
            r2 = s.find(searchstring, start)
            try:
                r1 = iter.next()
            except StopIteration:
                assert r2 == -1
                break
            assert r1 == r2
            start = r2 + max(len(searchstring), 1)

def test_hash():
    from rpython.rlib.rarithmetic import intmask
    for i in range(10):
        rope, _ = make_random_string()
        if rope.length() == 0:
            assert hash_rope(rope) == -1
            continue
        h = hash_rope(rope)
        x = LiteralStringNode(rope.flatten_string()).hash_part()
        assert x == rope.hash_part()
        x <<= 1
        x ^= rope.getint(0)
        x ^= rope.length()
        assert intmask(x) == h
        # hash again to check for cache effects
        h1 = hash_rope(rope)
        assert h1 == h

def test_hash_collisions_identifiers():
    hashes1 = {}
    hashes2 = {}
    cs = [""] + [chr(i) for i in range(256)]
    cs = "_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
    for i in range(50000):
        s = "".join([random.choice(cs)
                         for i in range(random.randrange(1, 15))])
        rope = LiteralStringNode(s)
        h1 = hash_rope(rope)
        hashes1[h1] = hashes1.get(h1, -1) + 1
        h2 = hash(s)
        hashes2[h2] = hashes2.get(h2, -1) + 1
    # hope that there are only ten percent more collisions
    # than with CPython's hash:
    assert sum(hashes1.values()) < sum(hashes2.values()) * 1.10

def test_hash_distribution_tiny_strings():
    hashes = [0 for i in range(256)]
    cs = [""] + [chr(i) for i in range(256)]
    for c1 in cs:
        for c2 in cs:
            rope = LiteralStringNode(c1 + c2)
            h = hash_rope(rope)
            hashes[h & 0xff] += 1
            hashes[(h & 0xff00) >> 8] += 1
            hashes[(h & 0xff0000) >> 16] += 1
    for h in hashes:
        assert h > 300

def test_hash_distribution_small_strings():
    random.seed(42) # prevent randomly failing test
    hashes = [0 for i in range(256)]
    for i in range(20000):
        s = "".join([chr(random.randrange(256))
                         for i in range(random.randrange(1, 15))])
        rope = LiteralStringNode(s)
        h = hash_rope(rope)
        hashes[h & 0xff] += 1
        hashes[(h & 0xff00) >> 8] += 1
        hashes[(h & 0xff0000) >> 16] += 1
    for h in hashes:
        assert h > 180
    print hashes

def test_hash_distribution_big_strings():
    random.seed(42) # prevent randomly failing test
    hashes = [0 for i in range(256)]
    for i in range(4000):
        s = "".join([chr(random.randrange(256))
                         for i in range(random.randrange(20, 500))])
        rope = LiteralStringNode(s)
        h = hash_rope(rope)
        hashes[h & 0xff] += 1
        hashes[(h & 0xff00) >> 8] += 1
        hashes[(h & 0xff0000) >> 16] += 1
    for h in hashes:
        assert h > 29

def test_hash_distribution_identifiers():
    random.seed(42) # prevent randomly failing test
    hashes = [0 for i in range(256)]
    cs = "_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
    for i in range(50000):
        s = "".join([random.choice(cs)
                         for i in range(random.randrange(1, 15))])
        rope = LiteralStringNode(s)
        h = hash_rope(rope)
        hashes[h & 0xff] += 1
        hashes[(h & 0xff00) >> 8] += 1
        hashes[(h & 0xff0000) >> 16] += 1
    for h in hashes:
        assert h > 450
    print hashes


def test_hash_part():
    a = "".join([chr(random.randrange(256)) * random.randrange(500)])
    h = None
    for split in range(1, len(a) - 1):
        s1 = LiteralStringNode(a[:split])
        s2 = LiteralStringNode(a[split:])
        s = BinaryConcatNode(s1, s2)
        if h is None:
            h = s.hash_part()
        else:
            # try twice due to caching reasons
            assert s.hash_part() == h
            assert s.hash_part() == h

def test_hash_part_unicode():
    a, st = make_random_string(5, unicode=True)
    h = a.hash_part()
    for split in range(1, len(st) - 1):
        s1 = LiteralUnicodeNode(st[:split])
        s2 = LiteralUnicodeNode(st[split:])
        s = BinaryConcatNode(s1, s2)
        if h is None:
            h = s.hash_part()
        else:
            # try twice due to caching reasons
            assert s.hash_part() == h
            assert s.hash_part() == h

def test_hash_part_more():
    for i in range(100):
        rope, st = make_random_string()
        h = rope.hash_part()
        assert LiteralStringNode(st).hash_part() == h

def test_equality():
    l = [make_random_string() for i in range(3)]
    l.append((LiteralStringNode(""), ""))
    for rope1, st1 in l:
        for rope2, st2 in l:
            assert eq(rope1, rope2) == (st1 == st2)

def test_equality_unicode():
    l = [make_random_string() for i in range(3)]
    l.append((LiteralStringNode(""), ""))
    for rope1, st1 in l:
        for rope2, st2 in l:
            assert eq(rope1, rope2) == (st1 == st2)

def test_compare_random():
    l = [make_random_string() for i in range(3)]
    l.append((LiteralStringNode(""), ""))
    for rope1, st1 in l:
        for rope2, st2 in l:
            c = compare(rope1, rope2)
            if c:
                c = c // abs(c)
            assert c == cmp(st1, st2)

def test_compare_random_unicode():
    l = [make_random_string() for i in range(3)]
    l.append((LiteralStringNode(""), ""))
    for rope1, st1 in l:
        for rope2, st2 in l:
            c = compare(rope1, rope2)
            if c:
                c = c // abs(c)
            assert c == cmp(st1, st2)


def test_power():
    for i in range(0, 60, 13):
        print i
        for j in range(1, 10000, 7):
            assert intmask(i * (1000003**j)) == masked_mul_by_1000003_pow(i, j)


def test_seekable_bug():
    node = BinaryConcatNode(LiteralStringNode("abc"), LiteralStringNode("def"))
    iter = SeekableItemIterator(node)
    c = iter.nextchar(); assert c == "a"
    c = iter.nextchar(); assert c == "b"
    c = iter.nextchar(); assert c == "c"
    iter.seekback(1)
    c = iter.nextchar(); assert c == "c"
    c = iter.nextchar(); assert c == "d"
    c = iter.nextchar(); assert c == "e"
    c = iter.nextchar(); assert c == "f"
    py.test.raises(StopIteration, iter.nextchar)
    node = LiteralStringNode("abcdef")
    iter = SeekableItemIterator(node)
    c = iter.nextchar(); assert c == "a"
    c = iter.nextchar(); assert c == "b"
    c = iter.nextchar(); assert c == "c"
    iter.seekback(3)
    c = iter.nextchar(); assert c == "a"
    c = iter.nextchar(); assert c == "b"
    c = iter.nextchar(); assert c == "c"

def test_strip():
    node = BinaryConcatNode(LiteralStringNode(" \t\n abc "),
                            LiteralStringNode("def    "))
    r = strip(node)
    assert r.flatten_string() == "abc def"
    r = strip(node, left=False)
    assert r.flatten_string() == " \t\n abc def"
    r = strip(node, right=False)
    assert r.flatten_string() == "abc def    "

    node = BinaryConcatNode(LiteralStringNode("aaaYYYYa"),
                            LiteralStringNode("abab"))
    predicate = lambda i: chr(i) in "ab"
    r = strip(node, predicate=predicate)
    assert r.flatten_string() == "YYYY"
    r = strip(node, left=False, predicate=predicate)
    assert r.flatten_string() == "aaaYYYY"
    r = strip(node, right=False, predicate=predicate)
    assert r.flatten_string() == "YYYYaabab"

def test_getrope():
    s1, result = make_random_string(200, unicode=True)
    s2 = s1.rebalance()
    for i in range(len(result)):
        for s in [s1, s2]:
            r = s.getrope(i)
            assert r.length() == 1
            assert r.getint(0) == s.getint(i)
            assert isinstance(r, LiteralNode)
            assert r.getint(0) >= 128 or isinstance(r, LiteralStringNode)
            assert r.getrope(0) is r

def test_split():
    seps = [(LiteralStringNode("a"), "a"), (LiteralStringNode("abc"), "abc"),
            (LiteralStringNode("d"), "d"), (LiteralStringNode(""), "")]
    l, strs = zip(*[(LiteralStringNode("x"), "x"),
                    (LiteralStringNode("xyz"), "xyz"),
                    (LiteralStringNode("w"), "w")])
    l = list(l)
    for s, st in seps:
        node = join(s, l)
        l2 = split(node, s)
        for n1, n2 in zip(l, l2):
            assert n1.flatten_string() == n2.flatten_string()
    for i in range(4):
        l1 = split(LiteralStringNode("ababababa"), LiteralStringNode("b"), i)
        l2 = "ababababa".split("b", i)
        assert len(l1) == len(l2)
        for n, s in zip(l1, l2):
            assert n.flatten_string() == s

def test_splitlines():
    seps = [(LiteralStringNode("\n"), "\n"), (LiteralStringNode("\r"), "\r"),
            (LiteralStringNode("\r\n"), "\r\n")]
    l, strs = zip(*[(LiteralStringNode("xafnarsp"), "xafnarsp"),
                    (LiteralStringNode("xyzaaaa"), "xyzaaaa"),
                    (LiteralStringNode("wxxxx"), "wxxxx")])
    l = list(l)
    for s, st in seps:
        node = join(s, l)
        l2 = splitlines(node)
        for n1, n2 in zip(l, l2):
            assert n1.flatten_string() == n2.flatten_string()
    for keepends in [True, False]:
        l1 = splitlines(LiteralStringNode("ab\nab\n\raba\rba"), keepends)
        l2 = "ab\nab\n\raba\rba".splitlines(keepends)
        assert len(l1) == len(l2)
        for n, s in zip(l1, l2):
            assert n.flatten_string() == s

def test_rope_from_unicode():
    node = rope_from_unicode(u"aaabbbbbcccdddeeefffggggnnn")
    assert node.is_bytestring()
    assert node.is_ascii()
    node = rope_from_unicode(u"a" * 30 + u"\ufffd" * 30 + "x" * 30)
    assert node.length() == 90
    assert not node.is_ascii()
    assert not node.is_bytestring()

def test_encode():
    node = LiteralStringNode("abc")
    assert unicode_encode_ascii(node) is node
    assert unicode_encode_latin1(node) is node
    assert unicode_encode_utf8(node) is node
    node = LiteralStringNode("abc\xff")
    assert unicode_encode_ascii(node) is None
    assert unicode_encode_latin1(node) is node
    assert unicode_encode_utf8(node).s == 'abc\xc3\xbf'
    node = LiteralUnicodeNode(u"\uffffab")
    assert unicode_encode_ascii(node) is None
    assert unicode_encode_latin1(node) is None
    assert unicode_encode_utf8(node).s == '\xef\xbf\xbfab'
    node = BinaryConcatNode(LiteralStringNode("abc"),
                            LiteralUnicodeNode(u"\uffffab"))
    assert unicode_encode_ascii(node) is None
    assert unicode_encode_latin1(node) is None
    res = unicode_encode_utf8(node)
    assert res.left is node.left
    assert res.right.s == '\xef\xbf\xbfab'

def test_decode():
    node = LiteralStringNode("abc")
    assert str_decode_ascii(node) is node
    assert str_decode_latin1(node) is node
    assert str_decode_utf8(node) is node
    node = LiteralStringNode("abc\xff")
    assert str_decode_ascii(node) is None
    assert str_decode_latin1(node) is node

def test_decode_utf8():
    # bad data
    node = LiteralStringNode("\xd7\x50")
    assert str_decode_utf8(node) is None
    node = LiteralStringNode("\xf0\x90\x91")
    assert str_decode_utf8(node) is None

    # correct data in one node
    node = LiteralStringNode('\xef\xbf\xbfab')
    assert str_decode_utf8(node).u == u"\uffffab"

    # binary node, left node can be decoded
    node = BinaryConcatNode(LiteralStringNode('\xef\xbf\xbfab'),
                            LiteralStringNode('\xef\xbf\xbfab'))
    res = str_decode_utf8(node)
    assert res.left.u == u"\uffffab"
    assert res.right.u == u"\uffffab"

    # binary node, left node alone cannot be decoded
    node = BinaryConcatNode(LiteralStringNode('\xef'),
                            LiteralStringNode('\xbf\xbfab'))
    res = str_decode_utf8(node)
    assert res.u == u"\uffffab"

    # binary node, left node cannot be decoded, bad data
    node = BinaryConcatNode(LiteralStringNode("\xf0\x90"),
                            LiteralStringNode("\x91"))
    assert str_decode_utf8(node) is None

    # binary node, incomplete data
    node = BinaryConcatNode(LiteralStringNode('ab\xef'),
                            LiteralStringNode('\xbf'))
    
    res = str_decode_utf8(node)
    assert res is None

def test_multiply_result_needs_no_rebalancing():
    r1 = multiply(LiteralStringNode("s"), 2**31 - 2)
    assert r1.rebalance() is r1
